/*
*  dns_lookup.c
*
*  deals with the dns lookup of servers stuff
*
*
*  (c) 2001 Tim-Philipp Muller <t.i.m@orange.net>
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "misc.h"
#include "dns_lookup.h"
#include "status_page.h"
#include "statusbar.h"
#include "gui_options.h"
#include "guicoreconn.h"
#include "send_command.h"

#include <unistd.h>

// --- LOCAL VARIABLES ---

static gint		 dns_lookup_single_lookup_status = -1;
static gboolean		 dns_lookup_in_progress = FALSE;	// is TRUE while we do lookup
static GPtrArray	*dns_lookup_list = NULL;		// contains pointers to GURL* structures
static GPtrArray	*dns_server_list = NULL;		// contains pointers to GInetAddr* after lookup

// --- LOCAL FUNCTIONS DECLARATIONS ---
static void	dns_lookup_free_lookuplist (void);
static void	dns_lookup_free_serverlist (void);


// --- FUNCTIONS ---


// dns_lookup_free_lookuplist
//
// frees all the memory taken up by the dns_lookup_servers GPtrArray and its records

static void
dns_lookup_free_lookuplist (void)
{
	if (dns_lookup_list)
	{
		guint i;
		for (i=0; i<dns_lookup_list->len; i++)
		{
			GURL *url = (GURL*) g_ptr_array_index (dns_lookup_list, i);
			if (url) gnet_url_delete (url);
		}
		g_ptr_array_free (dns_lookup_list, TRUE);
		dns_lookup_list = NULL;
	}
}


// dns_lookup_free_serverlist
//
// frees all memory and structures of the dns_lookup_server_list GPtrArray

static void
dns_lookup_free_serverlist (void)
{
	if (dns_server_list)
	{
		guint i;
		for (i=0; i<dns_server_list->len; i++)
		{
			GInetAddr *ia = (GInetAddr*) g_ptr_array_index (dns_server_list, i);
			if (ia) gnet_inetaddr_delete (ia);
		}
		g_ptr_array_free (dns_server_list, TRUE);
		dns_server_list = NULL;
	}
}

// dns_lookup_read_lookuplist
//
// read the gui_lookuplist file and put the servers in the GPtrArray dns_lookup_list

void
dns_lookup_read_lookuplist (void)
{
	gchar	**line, **linearray;

	dns_lookup_free_lookuplist ();
	dns_lookup_list = g_ptr_array_new();
	g_return_if_fail (dns_lookup_list!=NULL);

	linearray = misc_get_array_of_lines_from_textfile (complete_pref_path(N_("gui_lookuplist"),GUI_CONFIG_IGNORE_INSTANCE));
	if (!linearray) return;
	line = linearray;
	while (*line)
	{
		if (**line!='#' && **line!=0x00)
		{
			gchar *thisline = g_strchug(*line);	// remove leading whitespaces
			if (strlen(thisline)>5 && *thisline!='#')
			{
				gchar *urlstr;
				gchar *proto = N_("normal"); // XXX - we'll abuse that for 'priority'
				if (*thisline == '+')
				{
					proto = N_("high");
					thisline++;
				}
				urlstr = g_strdup_printf (N_("%s://%s"), proto, thisline);
				if (urlstr)
				{
					GURL *url;
					if ((url = gnet_url_new (urlstr)))
					{
						// set to default port if no port is specified
						if (!url->port) gnet_url_set_port (url, 4661);
						g_ptr_array_add (dns_lookup_list, url);
					}
					g_free (urlstr);
				}
			}
		}
		line++;
	}
	g_strfreev(linearray);	// free line array
}


// dns_lookup_remove_dns_servers
//
// Removes the server IPs/ports added via DNS lookup from core's serverlist

void
dns_lookup_remove_dns_servers (void)
{
	if ((dns_server_list) && (dns_server_list->len > 0))
	{
		guint i;
		statusbar_extra_message (N_("%s"), _(" Removing servers on dynamic ip..."));
		for (i=0; i<dns_server_list->len; i++)
		{
			GInetAddr	*ia = (GInetAddr*) g_ptr_array_index (dns_server_list, i);
			if (ia)
			{
				send_remove_server (misc_gnet_inetaddr_get_ip(ia), gnet_inetaddr_get_port(ia));
			}
			while (gtk_events_pending())
			{
				gtk_main_iteration();
			}
		
		}
	        statusbar_extra_message (N_("%s"), _(" Done removing DNS servers. (core shutdown may take a few seconds...)"));
	}
	dns_lookup_free_serverlist ();
}



// dns_lookup_callback
//
// The callback that is called when the lookup succeeded or failed
// data is: 0 for normal priority, 1 for high priority

void
dns_lookup_callback (GInetAddr *inetaddr, GInetAddrAsyncStatus status, gpointer data)
{
	guint priority = GPOINTER_TO_UINT(data);

	switch (status)
	{
		case GINETADDR_ASYNC_STATUS_OK:
		{
			guint32 ip = misc_gnet_inetaddr_get_ip (inetaddr);
			guint16 port = gnet_inetaddr_get_port (inetaddr);
			if (ip>0)
			{
				send_add_server (ip, port);
				// set server to high priority?
				if (priority==1) send_set_server_priority (ip, port, 1);
			}
			if (!dns_server_list) dns_server_list = g_ptr_array_new();
			if (dns_server_list) g_ptr_array_add (dns_server_list, (gpointer)inetaddr);
		}
		break;

		case GINETADDR_ASYNC_STATUS_ERROR:
		break;
	}
	dns_lookup_single_lookup_status = status;
}



// dns_lookup_do_lookup
//
// Start the DNS lookup procedure

void
dns_lookup_do_lookup (void)
{
	guint i;
	if (dns_lookup_in_progress) return;
//	XXX - Do NOT check whether core is alive. We call this function when we just
//		got connected to the core. At that point the core is technically not
//		'alive' yet (because status is not _COMMUNICATING yet).
//	if (!CORE_IS_ALIVE) return;		// if core dead we've got nothing to do
	if (!dns_lookup_list) return;
	if (dns_lookup_list->len==0) return;

	dns_lookup_free_serverlist ();
	dns_lookup_in_progress = TRUE;
	for (i=0; i<dns_lookup_list->len; i++)
	{
		guint		 priority = 0;
		GInetAddr	*inetaddr;

		GURL *url = g_ptr_array_index (dns_lookup_list, i);
		if (!url) continue;
		if (g_strncasecmp(url->protocol,N_("high"),4)==0) priority = 1; // high
		dns_lookup_single_lookup_status = -1;

		/* check number of CPUs at runtime, so we only call the slow DNS lookup code that
		 * will freeze the GUI if we are on multi-processor systems
		 */
		if (misc_get_ncpus()==1)
		{

		/*  DO ASYNCHRONEOUS LOOKUP. DOESN'T WORK ON SMP MACHINES DUE TO A BUG IN GLIB OR GNET.
			BUT AS WE WAIT FOR THE RESULT RIGHT AWAY ANYWAY (AND BLOCK THE DISPLAY), WE CAN
			JUST AS WELL DO A SYNCHRONEOUS LOOKUP... (thanks to Roman Hodek for fixing this problem)
		*/
			if (0 < gnet_inetaddr_new_async (url->hostname, url->port, dns_lookup_callback, GUINT_TO_POINTER(priority)))
			{
				while (dns_lookup_single_lookup_status == -1)
				{
					/*
					 * XXX - for some reasons (I suppose in connection with threads)
					 * we can NOT do a gtk_main_iteration() here without screwing up
					 * the GUI (everlasting freeze) (????)
					 */
/*					if (gtk_events_pending())
						gtk_main_iteration();
*/
					g_main_iteration(FALSE);
				}
			}

		} else {

			/* we are on a SMP-machine. This code is bad, but at least it doesn't crash */
			inetaddr = gnet_inetaddr_new (url->hostname, url->port);
			if (gtk_events_pending())
				gtk_main_iteration();
			dns_lookup_single_lookup_status =
					(inetaddr!=NULL) ? GINETADDR_ASYNC_STATUS_OK :
								GINETADDR_ASYNC_STATUS_ERROR;
			/* calling the old async callback is dirty, but works for now */
			dns_lookup_callback(inetaddr, dns_lookup_single_lookup_status, GUINT_TO_POINTER(priority));
		}

		gnet_url_set_resource (url, (dns_lookup_single_lookup_status==GINETADDR_ASYNC_STATUS_OK) ? N_("/OK") : N_("/FAIL"));
		status_msg (_("GUI: DNS lookup of %s:%u %s\n"), url->hostname, url->port, url->resource);
	}
	dns_lookup_in_progress = FALSE;
}

