/***************************************************************************
                          misc_gtk.c  -  description
                             -------------------
    begin                : Fri Nov 29 2002
    copyright            : (C) 2002 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "icons.h"
#include "mainwindow.h"
#include "misc_gtk.h"

/* ok_dialog             - display OK dialog with a text message
 * ok_dialog_from_widget - display OK dialog with a widget
 *
 * Displays a dialog with a title, a message, and an OK button.
 *
 */

static void
misc_gtk_ok_dialog_button_clicked (GtkWidget *widget, gpointer data)
{
	/* Okay button clicked. now destroy window    */
	/* Note: widget is button, data is the dialog */
	gtk_widget_destroy (GTK_WIDGET (data));
}

static gint
misc_gtk_ok_dialog_destroy (GtkWidget *widget, gpointer data)
{
        /* release grabbed focus */
        gtk_grab_remove (GTK_WIDGET (widget));
        /* let gtk know that it's okay to close the window */
        return FALSE;
}


void
ok_dialog_from_widget (const gchar *title, GtkWidget *widget)
{
	GtkWidget	*dialog, *ok_button;

	g_return_if_fail (title != NULL);

	/* create dialog box and trap closing signal (so we can release grabbed focus) */
	dialog = gtk_dialog_new ();
	g_return_if_fail (dialog != NULL);

	gtk_widget_show (dialog);
/*	gtk_window_set_modal(GTK_WINDOW(dialog),TRUE); */

	(void) gtk_window_set_title (GTK_WINDOW (dialog), title);
	(void) gtk_widget_set_uposition (GTK_WIDGET (dialog), 320, 240);

	(void) gtk_signal_connect (GTK_OBJECT (dialog),
	                           N_("destroy"),
	                           GTK_SIGNAL_FUNC (misc_gtk_ok_dialog_destroy),
	                           NULL);

	/* create ok button, add it to dialog and make it default button */
	ok_button = gtk_button_new_with_label (_("    OK    "));
	g_return_if_fail (ok_button != NULL);

	gtk_widget_show (ok_button);

	(void) gtk_signal_connect (GTK_OBJECT (ok_button),
	                           N_("clicked"),
	                           GTK_SIGNAL_FUNC (misc_gtk_ok_dialog_button_clicked),
	                           dialog);

	GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (ok_button);

	gtk_container_set_border_width (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 15);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->action_area), ok_button, FALSE, FALSE, 5);

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), widget, TRUE, TRUE, 5);

	/* make dialog window visible and let it grab the focus */
	gtk_grab_add (dialog);

}


void
ok_dialog (const gchar *title, const gchar *txt)
{
	GtkWidget	*label;

	g_return_if_fail (title != NULL);
	g_return_if_fail (txt != NULL);

	/* create label to hold message text */
	label = gtk_label_new (txt);
	g_return_if_fail (label != NULL);
	gtk_widget_show (label);
	gtk_misc_set_padding (GTK_MISC (label), 20, 20);

	ok_dialog_from_widget (title, label);
}



/* yesno_dialog : a choice1 / choice2 dialogue
 *
 * presents a dialog to the user with two choices (buttons)
 *  and then calls the callback associated with the choice.
 *
 */

static gint
misc_gtk_yesno_dialog_destroy (GtkWidget *widget, gpointer data)
{
        /* release grabbed focus */
        gtk_grab_remove (GTK_WIDGET (widget));
        /* let gtk know that it's okay to close the window */
        return FALSE;
}

void
yesno_dialog (	const gchar *title,
				const gchar *text,
				const gchar *yes_txt,
				const gchar *no_txt,
				void (*yes_func)(),
				void (*no_func)())
{
	GtkWidget	*dialog, *label;
	GtkWidget	*yes_button = NULL;
	GtkWidget	*no_button = NULL;

	g_return_if_fail (title   != NULL);
	g_return_if_fail (text    != NULL);
	g_return_if_fail (yes_txt != NULL);
	g_return_if_fail (no_txt  != NULL);
	g_return_if_fail (yes_func!= NULL);
	g_return_if_fail (no_func != NULL);

	/* create dialog box and trap closing signal (so we can release grabbed focus) */
	dialog = gtk_dialog_new ();
	gtk_signal_connect (GTK_OBJECT (dialog),
	                    N_("destroy"),
	                    GTK_SIGNAL_FUNC (misc_gtk_yesno_dialog_destroy),
	                    NULL);

	gtk_window_set_title (GTK_WINDOW (dialog), title);
	gtk_widget_set_uposition (GTK_WIDGET (dialog), 320, 240);

	/* create 1st button, add it to dialog */
	new_icon_button_pack_and_signal_connect (
		&yes_button,
		GTK_DIALOG(dialog)->action_area,
		yes_txt,
		GTK_SIGNAL_FUNC (yes_func),
		(gpointer)dialog,
		1,
		icon_ok
	);
	g_return_if_fail (yes_button!=NULL);
	GTK_WIDGET_SET_FLAGS (yes_button, GTK_CAN_DEFAULT);

	/* create 2nd button, add it to dialog and make it default button */
	new_icon_button_pack_and_signal_connect (
		&no_button,
		GTK_DIALOG(dialog)->action_area,
		no_txt,
		GTK_SIGNAL_FUNC (no_func),
		(gpointer)dialog,
		1,
		icon_cancel
	);
	g_return_if_fail (no_button!=NULL);
	GTK_WIDGET_SET_FLAGS (no_button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (no_button);

	/* create label to hold message text */
	label = gtk_label_new (text);
	gtk_misc_set_padding (GTK_MISC (label), 20, 20);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 5);
	gtk_widget_show (label);

	/* make dialog window visible and let it grab the focus */
	gtk_widget_show (dialog);
	gtk_grab_add (dialog);
}






/* new_button_pack_and_signal_connect
 *
 *  - creates a new button with a label
 *  - adds it to a pack box at the start or end (pack_start_flag = 1/0)
 *  - connects its 'clicked' signal with a function
 *  - and makes it visible
 *
 */

void
new_button_pack_and_signal_connect (	GtkWidget **button_ptr,
										GtkWidget *hbox,
										const gchar *label,
										void *click_func,
										gpointer func_data,
										gint pack_start_flag)
{
	g_return_if_fail (button_ptr!=NULL);
	g_return_if_fail (hbox!=NULL);
	g_return_if_fail (click_func!=NULL);
	g_return_if_fail (label!=NULL);

	*button_ptr = gtk_button_new_with_label (label);

	/* connect button to function */

	gtk_signal_connect (GTK_OBJECT(*button_ptr), N_("clicked"),
		            GTK_SIGNAL_FUNC (click_func), func_data);

	if (pack_start_flag)
	{
		gtk_box_pack_start (GTK_BOX (hbox), *button_ptr, FALSE, TRUE, 2);
	} else gtk_box_pack_end   (GTK_BOX (hbox), *button_ptr, FALSE, TRUE, 2);
	gtk_widget_show (*button_ptr);
}


/* new_icon_button_pack_and_signal_connect
 *
 */

void
new_icon_button_pack_and_signal_connect (	GtkWidget **button_ptr,
											GtkWidget *hbox,
			 								const gchar *text,
											void *click_func,
											gpointer func_data,
											gint pack_start_flag,
											gchar **icon)
{
	g_return_if_fail (button_ptr != NULL);
	g_return_if_fail (hbox != NULL);
	g_return_if_fail (text != NULL);
	g_return_if_fail (click_func != NULL);
	g_return_if_fail (icon != NULL);

	*button_ptr = gtk_button_new ();
	gtk_container_add (GTK_CONTAINER(*button_ptr), make_icon_and_label_hbox(text,icon));

	/* connect button to function */
	gtk_signal_connect (GTK_OBJECT(*button_ptr), N_("clicked"),
		            GTK_SIGNAL_FUNC (click_func), func_data);

	if (pack_start_flag)
	{
		gtk_box_pack_start (GTK_BOX (hbox), *button_ptr, FALSE, TRUE,2);
	} else gtk_box_pack_end (GTK_BOX (hbox), *button_ptr, FALSE, TRUE,2);
	gtk_widget_show (*button_ptr);
}


/* new_icon_toggle_button_pack_and_signal_connect
 *
 */

void
new_icon_toggle_button_pack_and_signal_connect (	GtkWidget **button_ptr,
													GtkWidget *hbox,
			 										const gchar *text,
													void *click_func,
													gpointer func_data,
													gint pack_start_flag,
													gchar **icon)
{
	g_return_if_fail (button_ptr != NULL);
	g_return_if_fail (hbox != NULL);
	g_return_if_fail (text != NULL);
	g_return_if_fail (click_func != NULL);
	g_return_if_fail (icon != NULL);

	*button_ptr = gtk_toggle_button_new ();
	gtk_container_add (GTK_CONTAINER(*button_ptr), make_icon_and_label_hbox(text,icon));

	/* connect button to function */
	gtk_signal_connect (GTK_OBJECT(*button_ptr),
	                    N_("toggled"),
	                    GTK_SIGNAL_FUNC(click_func),
	                    func_data);

	if (pack_start_flag)
	{
		gtk_box_pack_start (GTK_BOX (hbox), *button_ptr, FALSE, TRUE,2);
	} else gtk_box_pack_end (GTK_BOX (hbox), *button_ptr, FALSE, TRUE,2);
	gtk_widget_show (*button_ptr);
}


/* add_new_check_button_to_box
 *
 *
 */

void
add_new_check_button_to_box (GtkWidget *box, GtkWidget **cbwidget, const gchar *label)
{
	g_return_if_fail (box != NULL);
	g_return_if_fail (cbwidget != NULL);
	g_return_if_fail (label != NULL);

	*cbwidget = gtk_check_button_new_with_label (label);
	gtk_box_pack_start (GTK_BOX (box), *cbwidget, FALSE, FALSE, 2);
	gtk_widget_show (*cbwidget);
}


/* add_new_entry_widget_to_box
 *
 *
 *
 */

void
add_new_entry_widget_to_box (	GtkWidget *box,
								GtkWidget **entrywidget,
								const gchar *entrylabel,
								guint16 max_length )
{
	GtkWidget *hbox;		/* hbox to hold entry widget and label */
	GtkWidget *label, *dummy;

	hbox = gtk_hbox_new (FALSE,0);					/* orig: TRUE,0 */

	label = gtk_label_new (entrylabel);
	gtk_misc_set_padding (GTK_MISC (label),0,2);
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5); 			/* backup:was 0, not 0.0 */
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);		/* orig: FALSE TRUE */
	gtk_widget_show (label);

	*entrywidget = gtk_entry_new_with_max_length (max_length);
	gtk_box_pack_start (GTK_BOX (hbox), *entrywidget, FALSE, FALSE, 0);	/* orig: FALSE FALSE */
	gtk_widget_show (*entrywidget);

	dummy = gtk_label_new (N_("     "));
	gtk_box_pack_start (GTK_BOX (hbox), dummy, FALSE, FALSE, 5);		/* orig: FALSE TRUE */
	gtk_widget_show (dummy);

	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);	/* orig: FALSE FALSE --- add hbox to vbox */
	gtk_widget_show (hbox);
}


/* add_labelled_menu_widget_to_box
 *
 *
 *
 */

void
add_labelled_menu_widget_to_box (	GtkWidget *box,
									GtkWidget *menuwidget,
									const gchar *entrylabel )
{
	GtkWidget *hbox;		// hbox to hold entry widget and label
	GtkWidget *label, *dummy;

	hbox = gtk_hbox_new (FALSE,0);					// orig: TRUE,0

	label = gtk_label_new (entrylabel);
	gtk_misc_set_padding (GTK_MISC (label),0,2);
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5); 			// backup:was 0, not 0.0
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);		// orig: FALSE TRUE
	gtk_widget_show (label);

	gtk_box_pack_start (GTK_BOX (hbox), menuwidget, FALSE, FALSE, 0);	// orig: FALSE FALSE
	gtk_widget_show (menuwidget);

	dummy = gtk_label_new ("     ");
	gtk_box_pack_start (GTK_BOX (hbox), dummy, FALSE, FALSE, 5);		// orig: FALSE TRUE
	gtk_widget_show (dummy);

	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 5);	// orig: FALSE FALSE	// add hbox to vbox
	gtk_widget_show (hbox);
}


/* make_icon_and_label_hbox
 *
 * create pop-up menuitem, append it to menu, and make it visible
 *
 */

GtkWidget *
make_icon_and_label_hbox (const gchar *text, gchar **icon)
{
	GtkWidget *hbox=NULL, *pix, *label;

	g_return_val_if_fail (text!=NULL,NULL);
	g_return_val_if_fail (icon!=NULL,NULL);
	g_return_val_if_fail (window!=NULL,NULL);

	/* create hbox and stuff and pack stuff in there */
	hbox = gtk_hbox_new(FALSE,0);
	if (hbox)
	{
		if (icon) pix = create_widget_from_xpm (window, icon);
		else pix = create_widget_from_xpm (window, icon_none);
		if (pix)
		{
			gtk_widget_show(pix);
			gtk_box_pack_start (GTK_BOX(hbox), pix, FALSE,FALSE,0);
			label = gtk_label_new (text);
			gtk_widget_show(label);
			gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE,0);
			gtk_widget_show(hbox);
		}
	}
	return hbox;
}


