/*
 *  search_page.c
 *
 *  part of ed2k-gtk-gui
 *
 *  populates search page in notepad and deals with its functions
 *
 *  (C) 2001-2003 Tim-Philipp Mller <t.i.m@orange.net>
 *
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// TODO: adjust custom dialog so, that not all windows are displayed at the same
//		position, but slightly cascaded

#include <stdlib.h>

#include "colors.h"
#include "global.h"
#include "misc.h"
#include "misc_gtk.h"
#include "notebook.h"
#include "icons.h"
#include "blacklist.h"
#include "statusbar.h"
#include "status_page.h"
#include "search_page.h"
#include "gui_options.h"
#include "gui_flags.h"
#include "search_data.h"
#include "filter.h"
#include "send_command.h"
#include "parse_message.h"


/* global variables */

GtkWidget	*button_extend = NULL;		/* so we can activate it once we got results - thus global */
GtkWidget	*button_more = NULL;		/* ditto */
GtkWidget	*button_forget = NULL;		/* ditto */
GtkWidget	*button_addsearch = NULL;	/* ditto */

/* local variables */

static GString		*last_searchtype_selected = NULL;
static searchList	*active_searchlist = NULL;			/* the search list visible */
static GtkWidget	*ew_addsearch;
static GtkWidget	*minsize, *maxsize, *minavail, *minbitrate;		/* GtkSpinButton widgets */
static GtkWidget	*search_hbox_extra_options;
static gboolean		 search_has_changed = FALSE;
static GtkWidget	*om_search = NULL;

/* local functions */

static gint		searchclist_compare_func (GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2);
static gint		search_compare_func_high_to_low (gconstpointer a, gconstpointer b);

static void		create_scrolled_search_clist (GtkWidget **p_scrollwin, GtkWidget **p_newclist);
static gint		searchclist_button_pressed (GtkWidget *widget, GdkEvent *event, gpointer data);


/******************************************************************************
 *   searchclist_compare_func
 *
 *   compare function to sort the clist
 *
 */

static gint
searchclist_compare_func (GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
	gchar				*text1=NULL, *text2=NULL;
	searchlistRecord	*si1, *si2;

	g_return_val_if_fail (ptr1!=NULL,0);				/* just making sure... */
	g_return_val_if_fail (ptr2!=NULL,0);

	si1 = (searchlistRecord*) ((GtkCListRow*)ptr1)->data;
	si2 = (searchlistRecord*) ((GtkCListRow*)ptr2)->data;

	g_return_val_if_fail (si1!=NULL,0);					/* just making sure... */
	g_return_val_if_fail (si2!=NULL,0);

	switch (clist->sort_column)
	{
		/* column 0 -> filename     */
		case 0: MISC_RETURN_STR_COMPARE(si1->name,si2->name);

		/* column 1 -> size         */
		case 1: MISC_RETURN_NUM_COMPARE(si1->size,si2->size);

		/* column 2 -> availability */
		case 2: MISC_RETURN_NUM_COMPARE(si1->availability,si2->availability);

		/* column 3 -> type quark   */
		/* column 4 -> length quark */
		/* column 6 -> codec quark  */
		/* column 10 -> extension quark */
		case 3:
		case 4:
		case 6:
		case 10:
		break;		/* dealt with below */

		/* column 5 -> bitrate      */
		case 5: MISC_RETURN_NUM_COMPARE(si1->bitrate,si2->bitrate);

		/* column 7 -> artist       */
		case 7: MISC_RETURN_STR_COMPARE(si1->mp3_artist,si2->mp3_artist);

		/* column 8 -> title        */
		case 8: MISC_RETURN_STR_COMPARE(si1->mp3_title,si2->mp3_title);

		/* column 9 -> album        */
		case 9: MISC_RETURN_STR_COMPARE(si1->mp3_album,si2->mp3_album);

		/* column 11 -> hash */
		case 11: 
		{
			guint n1 = GUINT_FROM_BE(*((guint*)(si1->hash)));
			guint n2 = GUINT_FROM_BE(*((guint*)(si2->hash)));
			MISC_RETURN_NUM_COMPARE(n1,n2);
		}
		
		default:
		break;		/* dealt with below */
	}

	get_text_from_gtk_clist_rows ((GtkCListRow *) ptr1, (GtkCListRow *) ptr2, clist->sort_column, &text1, &text2);

	if (!text2)
		return (text1 != NULL);

	if (!text1)
		return -1;

	return g_strcasecmp (text1, text2);
}



/******************************************************************************
 *
 *   search_compare_func_high_to_low
 *
 *   a sort compare function for g_list_sort() that sorts from high to low
 *
 */

static gint
search_compare_func_high_to_low (gconstpointer a, gconstpointer b)
{
	if (GPOINTER_TO_UINT(a) > GPOINTER_TO_UINT(b))
		return -1;

	return (GPOINTER_TO_UINT(a) == GPOINTER_TO_UINT(b)) ? 0 : 1;
}


/******************************************************************************
 *
 *   searchclist_column_clicked
 *
 *   a sort compare function for g_list_sort() that sorts from high to low
 *
 */

static void
searchclist_column_clicked (GtkCList *clist, gint column, gpointer data)
{
	GtkSortType		new_sort_type = GTK_SORT_ASCENDING;

	g_return_if_fail (clist != NULL);
	g_return_if_fail (column >= 0);

	last_time_of_user_click = time(NULL);

	/* other column than that which is already active for sorting? -> change sort column */
	if (column != clist->sort_column)
	{
		/* set new sort column; default type = ascending */
		gtk_clist_set_sort_column (clist, column);
	} else new_sort_type = (clist->sort_type == GTK_SORT_ASCENDING) ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING;

	gtk_clist_set_sort_type (clist, new_sort_type);
	searchclist_resort(clist);

#warning **************************************************
#warning *
#warning * FIX ME (searchclist_column_clicked)
#warning *
#warning * - set searchlist_sortcolumn/searchlist_sorttype
#warning *   if it is the current search tab
#warning * - FIX search col/type SAVE stuff as well
#warning *
#warning **************************************************

}


/******************************************************************************
 *
 *   search_entry_widget_changed
 *
 *
 */

static void
search_entry_widget_changed (GtkWidget *widget, gpointer data)
{
	last_time_of_user_click = time(NULL);
	g_return_if_fail (button_extend != NULL);
	gtk_widget_set_sensitive (button_extend, FALSE);
	search_has_changed = TRUE;
}


/******************************************************************************
 *
 *  search_from_toolbar
 *
 *
 */

void
search_from_toolbar (const gchar *searchphrase)
{
	g_return_if_fail (searchphrase!=NULL);

	searchlist_make_new_searchlist (searchphrase);

	if (!IS_OVERNET)
	{
		gtk_widget_set_sensitive (button_extend, FALSE);
		gtk_widget_set_sensitive (button_more, FALSE);
		gtk_widget_set_sensitive (button_addsearch, FALSE);
	} else {
		gtk_widget_hide (button_extend);
		gtk_widget_hide (button_more);
		gtk_widget_set_sensitive (button_addsearch, TRUE);
	}

	gtk_entry_set_text (GTK_ENTRY(ew_addsearch), searchphrase);
	gtk_option_menu_set_history (GTK_OPTION_MENU(om_search), 0);

	send_search (searchphrase);
}


// search_add_search_button_clicked
//
// Note: data is search phrase entry widget (we don't need that though)

static void
search_add_search_button_clicked (GtkWidget *widget, gpointer data)
{
	gchar		*searchphrase;
	gchar		*searchtype_english = NULL;
	gboolean	 search_with_extra_options = GTK_WIDGET_VISIBLE(search_hbox_extra_options);
	gfloat		 f_minsize, f_maxsize;	/* size in MB */
	/* we only initialise these in order to get rid of the compiler warning */
	guint		 i_minsize=0, i_maxsize=2147483647, i_minavail=0, i_minbitrate=0;

	g_return_if_fail (ew_addsearch!=NULL);

	searchphrase = (gchar *) gtk_editable_get_chars (GTK_EDITABLE(ew_addsearch), 0, -1);
	g_return_if_fail (searchphrase!=NULL);

	last_time_of_user_click = time(NULL);

	if (g_strcasecmp (searchphrase, DEFAULT_SEARCH_PHRASE) == 0)
	{
		statusbar_extra_message (_(" You have to enter a search term first."));
		g_free (searchphrase);
		return;
	}

	/* if we have extra search options active, check if these are not all set to
	 * their default values, meaning that we don't really have these active
	 */
	if (search_with_extra_options)
	{
		gboolean audiosearch = GTK_WIDGET_SENSITIVE(minbitrate);
		gboolean minbratdefault;
		f_minsize = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(minsize));
		i_minsize = (guint) MIN(f_minsize*1024.0*1024.0, 4294967295.0);
		f_maxsize = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(maxsize));
		i_maxsize = (guint) MIN(f_maxsize*1024.0*1024.0, 4294967295.0);
		i_minavail = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(minavail));
		i_minbitrate = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(minbitrate));
		minbratdefault = (audiosearch==FALSE) ? TRUE : i_minbitrate==128;
		if (i_minsize==0 && i_maxsize==(guint)0xffffffff && i_minavail==1 && (minbratdefault))
			search_with_extra_options = FALSE;
	}

	// only create a new notebook page if the phrase
	// or type has been changed
	if (search_has_changed == TRUE)
		searchlist_make_new_searchlist (searchphrase);

	search_has_changed = FALSE;

	gtk_widget_set_sensitive (button_extend, FALSE);
	gtk_widget_set_sensitive (button_more, FALSE);

	if (!IS_OVERNET)
	{
		gtk_widget_set_sensitive (button_addsearch, FALSE);
	}

	g_return_if_fail (last_searchtype_selected!=NULL);

	// here comes the action...

	/* simple search? */
	if (strncmp (last_searchtype_selected->str, _("Any"),3) == 0)
	{
		if (!search_with_extra_options)
		{
			send_search (searchphrase);
		} else send_search_with_extra_options (searchphrase, i_minsize, i_maxsize, i_minavail);
		statusbar_extra_message (_(" Searching..."));
		g_free (searchphrase);
		return;
	}

	/* user search? */
	if (strncmp(last_searchtype_selected->str, _("User"), 4) == 0)
	{
		send_search_user (searchphrase);
		current_search->isUserList = TRUE;
		statusbar_extra_message (_(" Searching for user..."));
		g_free (searchphrase);
		return;
	}

	/* typed search - take into account internationalisaton! */

	if (g_strcasecmp(last_searchtype_selected->str, _("Audio"))==0)
		searchtype_english = N_("Audio");

	if (g_strcasecmp(last_searchtype_selected->str, _("Video"))==0)
		searchtype_english = N_("Video");

	if (g_strcasecmp(last_searchtype_selected->str, _("Images"))==0)
		searchtype_english = N_("Image");

	if (g_strcasecmp(last_searchtype_selected->str, _("Programs"))==0)
		searchtype_english = N_("Pro");

	if (g_strcasecmp(last_searchtype_selected->str, _("Documents"))==0)
		searchtype_english = N_("Documents");

	if (!search_with_extra_options)
	{
		send_type_search (searchphrase, searchtype_english, i_minbitrate);
	} else send_type_search_with_extra_options (searchphrase, searchtype_english, i_minsize, i_maxsize, i_minavail, i_minbitrate);
	statusbar_extra_message (_(" Searching..."));
	g_free (searchphrase);
}




// search_option_menu_item_selected
//
//

static void
search_option_menu_item_selected (GtkWidget *widget, gpointer data)
{
	// N.B. each click on an item results in two calls to this function: first from the
	//	previously selected menu item widget, then from the newly selected menu item
	//	widget. In our case that doesn't matter.

	gchar *etxt;

	g_return_if_fail (data!=NULL);

	search_has_changed = TRUE;

	if (strcmp (last_searchtype_selected->str,(gchar*)data) != 0)
	{
		gtk_widget_set_sensitive (button_extend, FALSE);
		gtk_widget_set_sensitive (minbitrate, (g_strcasecmp((gchar*)data, _("Audio"))==0) );
		last_searchtype_selected = g_string_assign (last_searchtype_selected, (gchar*)data);
	}
	etxt = gtk_editable_get_chars (GTK_EDITABLE(ew_addsearch), 0, -1);
	g_return_if_fail (etxt!=NULL);
	if (etxt[0]=='<') gtk_entry_set_text (GTK_ENTRY(ew_addsearch), "");
	g_free(etxt);
}


static void
searchclist_column_resized (GtkCList *clist, int col, int new_width, gpointer data)
{
	last_time_of_user_click = time(NULL);
	column_widths[SEARCH_COLS][col] = new_width;
}



static void
search_show_filter_button_clicked (GtkWidget *widget, gpointer gdata)
{
	guint i;

	if (filters)
	{
		GString	*str = g_string_new(_("Active filters:\n\n"));

		g_return_if_fail (str!=NULL);

		for (i=0; i<no_of_filters; i++)
		{
			gchar *filterword = filters[i];
			if (!filterword) continue;
			if (*filterword=='*')
			{
				str = g_string_append (str, filterword+1);
				str = g_string_append (str, _(" (whole word)\n"));
			} else {
				str = g_string_append (str, filterword);
				str = g_string_append (str, _(" (substring)\n"));
			}
		}
		ok_dialog (_("show active filters"), str->str);
		g_string_free (str,TRUE);
	} else statusbar_extra_message (_(" No filters?!"));
}


// dialogue window with entry widget for 'download under custom name

//filelist_item *search_custom_dialog_item = NULL;
//GtkWidget *ew_search_custom;

static void
search_custom_dialog_destroy_dialog (searchCustomDialogStruct *scds)
{
	g_return_if_fail (scds!=NULL);
	if (scds->dialog)
		gtk_widget_destroy(scds->dialog);
	scds->dialog = NULL;
}

// data is searchCustomDialogStruct
static void
search_custom_dialog_ok_button_clicked (GtkWidget *widget, gpointer data)
{
	searchCustomDialogStruct	*scds = (searchCustomDialogStruct*) data;
	gchar						*customname;

	g_return_if_fail (scds!=NULL);
	g_return_if_fail (scds->slr!=NULL);

	customname = gtk_editable_get_chars (GTK_EDITABLE(scds->ew_search_custom), 0, -1);
	g_return_if_fail (customname!=NULL);
	send_new_download ((scds->slr)->hash, (scds->slr)->size, customname);
	statusbar_extra_message (_(" Item added to download list with custom file name."));
	search_custom_dialog_destroy_dialog (scds);
	g_free (customname);
}


// data is searchCustomDialogStruct
static gint
search_custom_dialog_destroy (GtkWidget *widget, gpointer data)
{
	searchCustomDialogStruct *scds = (searchCustomDialogStruct*) data;
	g_return_val_if_fail (scds!=NULL, FALSE);
//	gtk_grab_remove (scds->dialog);		// release grabbed focus
	g_free(scds);
	return FALSE;						// let gtk know that it's okay to close the window
}

static void
search_custom_dialog (searchlistRecord *slr)
{
	GtkWidget					*ok_button, *label; // *dialog, 
	gchar						*txt;
	searchCustomDialogStruct	*scds = g_new0(searchCustomDialogStruct,1);

	g_return_if_fail (slr!=NULL);
	g_return_if_fail (slr->name!=NULL);
	g_return_if_fail (scds!=NULL);

	scds->slr = slr;

//	search_custom_dialog_item = search_data_of_last_row_selected;
	last_time_of_user_click = time(NULL);

	// create dialog box and trap closing signal (so we can release grabbed focus)
	scds->dialog = gtk_dialog_new ();
	gtk_signal_connect (GTK_OBJECT (scds->dialog), N_("destroy"),
					GTK_SIGNAL_FUNC (search_custom_dialog_destroy), (gpointer) scds);
	gtk_window_set_title (GTK_WINDOW (scds->dialog), _(" download file with custom filename "));
	gtk_widget_set_uposition (GTK_WIDGET (scds->dialog), 320, 240);

	// create entry widget
	scds->ew_search_custom = gtk_entry_new_with_max_length (512);
	gtk_entry_set_text (GTK_ENTRY (scds->ew_search_custom), slr->name);
	gtk_signal_connect (GTK_OBJECT (scds->ew_search_custom), N_("activate"),
					GTK_SIGNAL_FUNC (search_custom_dialog_ok_button_clicked), (gpointer) scds);
	/* make num block enter key behave like normal enter key */
	gtk_signal_connect (GTK_OBJECT (scds->ew_search_custom), N_("insert_text"),
				GTK_SIGNAL_FUNC (misc_gtk_entry_insert_text_handler), (gpointer) scds);

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(scds->dialog)->action_area), scds->ew_search_custom, TRUE, TRUE,2);
	gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG(scds->dialog)->action_area), FALSE);
	gtk_widget_show (scds->ew_search_custom);
	gtk_widget_grab_focus (scds->ew_search_custom);

	// create ok button, add it to dialog and make it default button
	ok_button = gtk_button_new_with_label (_("    OK    "));
	gtk_signal_connect (GTK_OBJECT (ok_button), N_("clicked"),
					GTK_SIGNAL_FUNC (search_custom_dialog_ok_button_clicked), (gpointer) scds);
	GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (ok_button);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(scds->dialog)->action_area), ok_button, FALSE, FALSE, 5);
	gtk_widget_show (ok_button);

	// create label to hold message text
	txt = g_strdup_printf (_("Download this file under a filename of your choice:\n\n%s"), slr->name);
	label = gtk_label_new (txt);
	gtk_misc_set_padding (GTK_MISC (label), 20, 20);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(scds->dialog)->vbox), label, TRUE, TRUE, 5);
	gtk_widget_show (label);

	// make dialog window visible and let it grab the focus
	gtk_widget_show (scds->dialog);
}


// search_popupmenu_item_selected
//
// An item from the search pop-up menu has been selected. Take some action...
//
// data contains the menuitem selected (lowest 8 bits), and in some cases the column
//	where the click occured (second-lowest 8 bits), and some flags (second-highest 8 bits)
// Note: widget might be NULL

static void
search_popupmenu_item_selected (GtkWidget *widget, gpointer data)
{
	guint		menuitem =  GPOINTER_TO_UINT (data) & 0xff;
//	guint		column   = (GPOINTER_TO_UINT (data) & 0xff00) >> 8;
//	guint		flags    =  GPOINTER_TO_UINT (data) & 0xff0000;
	gboolean	custom_dl = FALSE;
	gboolean	unselect_all = TRUE;
	searchList	*sl = active_searchlist;

	g_return_if_fail (sl!=NULL);
	g_return_if_fail (sl->clist!=NULL);

	switch (menuitem)
	{
		case SL_MENUITEM_SELECT_ALL:
			gtk_clist_select_all (sl->clist);
			unselect_all = FALSE;
		break;

		case SL_MENUITEM_CLEAR_SELECTION:
			gtk_clist_unselect_all (sl->clist);
		break;

		case SL_MENUITEM_CUSTOM_DL_SELECTED:
			custom_dl = TRUE;
		// Note: NO BREAK HERE!

		case SL_MENUITEM_DL_SELECTED:
		{
			if ((sl->clist)->selection)
			{
				PROCESS_CLIST_SELECTION_LIST( sl->clist, sl->results, searchlistRecord,
					if (item)
					{
						if (custom_dl)
							 search_custom_dialog(item);
						else send_new_download (item->hash, item->size, item->name);
					}
				);
				if (!custom_dl) statusbar_extra_message (_(" Selected items added to download list."));
			}
		}
		break;

		case SL_MENUITEM_COPY_LINK_TO_CLIPBOARD:
		{
			status_msg (N_(" \n"));	// to make sure selection is only one link, for cut'n'paste to
								// fake search engines this is easier (if only one file was selected in list, that is);
			if ((sl->clist)->selection)
			{
				PROCESS_CLIST_SELECTION_LIST( sl->clist, sl->results, searchlistRecord,
					if (item) status_msg_print_ed2klink (item->name, item->size, item->hash);
				);
				statusbar_extra_message (_(" ed2k-link of this file written to logfile 'gui_statuslog' and to status log."));
			}
		}
		break;

		case SL_MENUITEM_SHOW_ALT_NAMES:
		{
			if ((sl->clist)->selection)
			{
				PROCESS_CLIST_SELECTION_LIST( sl->clist, sl->results, searchlistRecord,
					const gchar	*title = _("names file is known under on other servers");
					gchar		*txt;
					if ((item) && (item->altnames))
					{
						GString	*tmpstr = g_string_new(item->name);
						GSList	*node = item->altnames;
						tmpstr = g_string_append(tmpstr, N_(":\n\n"));
						while (node)
						{
							if (node->data)
							{
								tmpstr = g_string_append(tmpstr, (gchar*)node->data);
								tmpstr = g_string_append_c(tmpstr, '\n');
							}
							node=node->next;
						}
						txt = tmpstr->str;
						g_string_free(tmpstr, FALSE);	// free GString structure, but not gchar content
					} else txt = g_strdup_printf (_("%s\n\nI'm not aware of any alternative names for this file.\n"), item->name);
					ok_dialog ((gchar*)title, txt);
					g_free(txt);
				);
			} 
		}
		break;

		case SL_MENUITEM_ADD_TO_BLACKLIST:
		{
			if ((sl->clist)->selection)
			{
				// We need to sort the selection to contain the highest rows
				//	first, otherwise deleting a row will mess up the rest of
				//	the selection, as the selection contains row numbers, but
				//	removing a row will invalidate these...
				GList *selection_sorted = g_list_copy((sl->clist)->selection);
				selection_sorted = g_list_sort (selection_sorted, search_compare_func_high_to_low);

				gtk_clist_freeze(sl->clist);
				PROCESS_CLIST_SELECTION_LIST_CUSTOM_LIST( sl->clist, selection_sorted, sl->results, searchlistRecord,
					if (item)
					{
						add_to_blacklist (item->hash, item->size, item->name);
						/* make grey if option is set (fix by Roman Hodek) */
						if (!option_show_blacklist_items_grey)
							searchlist_remove_record (sl, item);
						else gtk_clist_set_foreground (sl->clist, row, &grey);
					}
				);
				statusbar_extra_message (_(" Selected items added to blacklist."));
				gtk_clist_thaw(sl->clist);
				g_list_free(selection_sorted);
			}
		}
		break;

		case SL_MENUITEM_SHOW_ACTIVE_FILTERS:
			search_show_filter_button_clicked (NULL, NULL);
		break;

		case SL_MENUITEM_ROTATE_TABS:
		{
			switch (option_search_notebook_tabs_pos)
			{
				case GTK_POS_LEFT:		option_search_notebook_tabs_pos = GTK_POS_TOP;
				break;
				case GTK_POS_RIGHT:		option_search_notebook_tabs_pos = GTK_POS_BOTTOM;
				break;
				case GTK_POS_TOP:		option_search_notebook_tabs_pos = GTK_POS_RIGHT;
				break;
				case GTK_POS_BOTTOM:	option_search_notebook_tabs_pos = GTK_POS_LEFT;
				break;
			}
			if (search_notebook)
				gtk_notebook_set_tab_pos(GTK_NOTEBOOK(search_notebook), (GtkPositionType)option_search_notebook_tabs_pos);
		}
		break;

		case SL_MENUITEM_GET_NEXT_200:
		{
			if (ew_addsearch)
			{
				gchar *txt = gtk_editable_get_chars (GTK_EDITABLE(ew_addsearch), 0, -1);
				if ((!txt) || ((txt) && (*txt==0x00)))
				{
					statusbar_extra_message (_(" Warning: 'More search' doesn't work with empty string searches (core/server bug).\n"
											 "         Search for file extension to circumvent this."));
				} else {
					statusbar_extra_message (_(" Okay, asked server for 200 more results."));
					gtk_widget_set_sensitive (button_addsearch, FALSE);
					send_more_search();
				}	
				if (txt) g_free(txt);
			}
		}
		break;

		case SL_MENUITEM_EXTEND_SEARCH_5X:
		{
			gint i;
			for (i=0; i<5; i++)
			{
				if ((active_searchlist) && (active_searchlist->isUserList))
					send_extended_search_user();
				else send_extended_search();
			}
		}
		break;

		case SL_MENUITEM_FORGET_SEARCH:
		{
			if ((searchlists) && (active_searchlist))
			{
				// if there is only one searchlist, create a new 
				// empty one and destroy the existing one
				if (searchlists->len == 1)
				{
					searchlist_destroy (active_searchlist);	// sl contains former active_searchlist
					searchlist_make_new_searchlist (_("empty"));
				}

				if (searchlists->len >= 2)
					searchlist_destroy (active_searchlist);
			}
		}
		break;

		case SL_MENUITEM_SHOW_OWN_FILES:
			option_show_own_files_in_search = (~option_show_own_files_in_search) & 1;	// XOR 1
		break;


		case SL_MENUITEM_TOGGLE_SHOW_BL_ITEMS:
			option_show_blacklist_items_grey = (~option_show_blacklist_items_grey) &1;	// XOR 1
		break;

		case SL_MENUITEM_RESET:
		{
			if (GTK_WIDGET_VISIBLE(search_hbox_extra_options))
			{
				gtk_spin_button_set_value (GTK_SPIN_BUTTON(minsize), 0.0);
				gtk_spin_button_set_value (GTK_SPIN_BUTTON(maxsize), 4096.0);
				gtk_spin_button_set_value (GTK_SPIN_BUTTON(minavail), 1.0);
				gtk_spin_button_set_value (GTK_SPIN_BUTTON(minbitrate), 128.0);
			}
			gtk_entry_set_text (GTK_ENTRY(ew_addsearch), "");
		}
		break;

		case SL_MENUITEM_TOGGLE_SEARCHOPT:
		{
			if (GTK_WIDGET_VISIBLE(search_hbox_extra_options))
			{
				gtk_widget_hide (search_hbox_extra_options);
			} else gtk_widget_show (search_hbox_extra_options);
			// we switch pages quickly so the widgets get resized. I didn't find a way
			// to do this otherwise. Somehow gtk_container_set_resize_mode to immediate
			// and gtk_container_check_resize does not work :-/
			gtk_notebook_prev_page (GTK_NOTEBOOK(notebook));
			gtk_notebook_next_page (GTK_NOTEBOOK(notebook));
			option_show_extra_search_options = (gint) GTK_WIDGET_VISIBLE(search_hbox_extra_options);
		}
		break;

		default:
			g_printerr (N_("GUI: Fix me - called with unknown value %u (%s)\n"), menuitem, __FUNCTION__);
		break;
	}

	last_time_of_user_click = time(NULL);
}



// search_popup_options_menu
//
// Creates the options submenu for the pop-up menu for the search list
// row col:  clist row and column where click took place

static GtkWidget *
search_popup_options_menu (guint row, guint col, GtkCList *clist, searchList *sl)
{
	GtkWidget *menu = NULL;

	const gchar	*SHOW_OWN_FILES =		_(" show files I have among search results (in next search)");
	const gchar	*SHOW_BL_ITEMS_GREY =	_(" show items on blacklist in grey (instead of hiding them)");
	const gchar	*ROTATE_TABS =			_(" change the search notebook tabs' position (rotate clockwise)");

	gchar		**icon_yes = icon_optionon;
	gchar		**icon_no  = icon_optionoff;

	menu  = gtk_menu_new();

	(void)	MENU_ITEM_ADD (SHOW_OWN_FILES,	search_popupmenu_item_selected, SL_MENUITEM_SHOW_OWN_FILES, (option_show_own_files_in_search==1) ? icon_yes : icon_no );
	(void)	MENU_ITEM_ADD (SHOW_BL_ITEMS_GREY, search_popupmenu_item_selected, SL_MENUITEM_TOGGLE_SHOW_BL_ITEMS, (option_show_blacklist_items_grey==1) ? icon_yes : icon_no);
			MENU_ITEM_ADD__________;
	(void)	MENU_ITEM_ADD (ROTATE_TABS, search_popupmenu_item_selected, SL_MENUITEM_ROTATE_TABS, icon_refresh);
	return menu;
}




// search_popup_menu
//
//

void
search_popup_menu (GtkCList *clist, searchList *sl, guint row, guint col, guint button, guint32 activate_time)
{
	GtkWidget	*menu = NULL;
	GtkWidget	*mi_nosel[25];	// items insensitive on no selection (NULL-terminated)
	GtkWidget	*mi_ul[25];		// items insensitive if it's a user list
	GtkWidget	*mi_fl[25];		// items insensitive if it's a file list
	GtkWidget	*optitem, *more=NULL, *ext5x=NULL, *forgets = NULL;
	gint nosel=0, ul=0, fl=0;

	const gchar	*DOWNLOAD_SELECTED = 		_(" download selected items");
	const gchar	*CUSTOM_DOWNLOAD_SELECTED =	_(" download selected items with custom filenames");
	const gchar	*SELECT_ALL = 				_(" select all");
	const gchar	*UNSELECT_ALL = 			_(" unselect all");
	const gchar	*COPY_LINK_TO_CLIPBOARD =	_(" write ed2k-link to log/clipboard ");
	const gchar	*SHOW_ALTERNATIVE_NAMES = 	_(" show names file is known under on other servers ");
	const gchar	*ADD_TO_BLACKLIST = 		_(" add item to blacklist (by hash+size) ");
	const gchar	*GET_NEXT_200 =				_(" get the next 200 search results ");
	const gchar	*EXTEND_SEARCH_5X =			_(" extend search to five other servers ");
	const gchar	*SHOW_ACTIVE_FILTERS = 		_(" show list of active filters (temporary) ");
	const gchar	*FORGET_SEARCH =			_(" forget this search ");
	const gchar	*SEARCH_OPTIONS =			_(" search list options ");

	GtkWidget			**menuwidget;

	g_return_if_fail (clist!=NULL);
	g_return_if_fail (sl!=NULL);


	menu  = gtk_menu_new();
	mi_nosel[nosel++] =	MENU_ITEM_ADD (SELECT_ALL, search_popupmenu_item_selected, SL_MENUITEM_SELECT_ALL, icon_none);
	mi_nosel[nosel++] =	MENU_ITEM_ADD (UNSELECT_ALL, search_popupmenu_item_selected, SL_MENUITEM_CLEAR_SELECTION, icon_none);
	(void)				MENU_ITEM_ADD__________;
	mi_nosel[nosel++] =
	mi_ul[ul++] =		MENU_ITEM_ADD (DOWNLOAD_SELECTED, search_popupmenu_item_selected, SL_MENUITEM_DL_SELECTED, icon_download);
	mi_nosel[nosel++] =
	mi_ul[ul++] =		MENU_ITEM_ADD (CUSTOM_DOWNLOAD_SELECTED, search_popupmenu_item_selected, SL_MENUITEM_CUSTOM_DL_SELECTED, icon_download);
	(void)				MENU_ITEM_ADD__________;
	mi_nosel[nosel++] =
	mi_ul[ul++] =		MENU_ITEM_ADD (COPY_LINK_TO_CLIPBOARD, search_popupmenu_item_selected, SL_MENUITEM_COPY_LINK_TO_CLIPBOARD, icon_status);
	mi_nosel[nosel++] =
	mi_ul[ul++] =		MENU_ITEM_ADD (SHOW_ALTERNATIVE_NAMES, search_popupmenu_item_selected, SL_MENUITEM_SHOW_ALT_NAMES, icon_about);
	(void)				MENU_ITEM_ADD__________;
	mi_nosel[nosel++] =
	mi_ul[ul++] =		MENU_ITEM_ADD (ADD_TO_BLACKLIST, search_popupmenu_item_selected, SL_MENUITEM_ADD_TO_BLACKLIST, icon_skull);
	(void)				MENU_ITEM_ADD__________;
	more =				MENU_ITEM_ADD (GET_NEXT_200, search_popupmenu_item_selected, SL_MENUITEM_GET_NEXT_200, icon_200more);
	ext5x =				MENU_ITEM_ADD (EXTEND_SEARCH_5X, search_popupmenu_item_selected, SL_MENUITEM_EXTEND_SEARCH_5X, icon_extend);
	(void)				MENU_ITEM_ADD__________;

	if (1==1)
	{
		static gboolean msg_shown = FALSE;
		if (!msg_shown)
			g_print (N_("IMPLEMENT: user stuff in search pop-up menu (current core doesn't support it though)\n"));
		msg_shown = TRUE;
//		item1 =  add_menuitem (menu, " show this user's files", search_menuitem_showuserfiles, NULL, icon_none);
//		item1b = add_menuitem (menu, " add user to friends list", search_menuitem_addtofriends, NULL, icon_none);
//		gtk_widget_set_sensitive (item1b, (GTK_CLIST(clist_search)->selection != NULL) );
	}

	(void)				MENU_ITEM_ADD (SHOW_ACTIVE_FILTERS, search_popupmenu_item_selected, SL_MENUITEM_SHOW_ACTIVE_FILTERS, icon_none);
	(void)				MENU_ITEM_ADD__________;
	forgets =			MENU_ITEM_ADD (FORGET_SEARCH, search_popupmenu_item_selected, SL_MENUITEM_FORGET_SEARCH, icon_cancel);
	(void)				MENU_ITEM_ADD__________;

	optitem	=			MENU_ITEM_ADD (SEARCH_OPTIONS, NULL, 0, icon_options);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM(optitem), search_popup_options_menu(row,col, clist, sl));

	mi_nosel[nosel++] = NULL;
	mi_ul[ul++] = NULL;
	mi_fl[fl++] = NULL;

	gtk_widget_set_sensitive (more, GTK_WIDGET_SENSITIVE(button_more));
	gtk_widget_set_sensitive (ext5x, GTK_WIDGET_SENSITIVE(button_extend));
	gtk_widget_set_sensitive (forgets, GTK_WIDGET_SENSITIVE(button_forget));

	menuwidget = mi_nosel;
	while (*menuwidget)
	{
		gtk_widget_set_sensitive (*menuwidget, clist->selection!=NULL);
		menuwidget++;
	}
	menuwidget = mi_ul;
	while (*menuwidget)
	{
		gtk_widget_set_sensitive (*menuwidget, (!sl->isUserList));
		menuwidget++;
	}
	menuwidget = mi_fl;
	while (*menuwidget)
	{
		gtk_widget_set_sensitive (*menuwidget, (sl->isUserList));
		menuwidget++;
	}
	// set all items insensitive if no selection (overrides criteria before) - XXX ???
	menuwidget = mi_nosel;
	while (*menuwidget)
	{
		if (GTK_WIDGET_IS_SENSITIVE(*menuwidget))
			gtk_widget_set_sensitive (*menuwidget, clist->selection!=NULL);
		menuwidget++;
	}

	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, activate_time);
}



/******************************************************************************
 *
 *   searchclist_button_pressed
 *
 *   right-click pop-up menu code by Mike Hicks, found on the gtk-list@redhat.com
 *   mailing list on 25/11/98
 */

static gint
searchclist_button_pressed (GtkWidget *widget, GdkEvent *event, gpointer data)
{
	GdkEventButton *bevent = (GdkEventButton *) event;
	GtkCList       *clist = NULL;
	GList          *selection = NULL; 
	gint            row = -1;
	gint            col = -1;

	last_time_of_user_click = time(NULL);

	g_return_val_if_fail (active_searchlist != NULL, 0);

	clist = active_searchlist->clist;
	g_return_val_if_fail (clist != NULL, 0);

	selection = clist->selection;

	gtk_clist_get_selection_info (clist, (int)bevent->x ,(int)bevent->y, &row, &col);

	/* single right click? */
	if (event->type == GDK_BUTTON_PRESS  &&  bevent->button == 3)
	{
		if (row >= 0)
		{
			/* no items selected => select the one where we right-clicked */
			if (selection == NULL)
			{
				gtk_clist_select_row (clist, row, col);
			}
			/* one item selected => unselect it and select
			 * the one where we right-clicked
			 */
			else if (g_list_length(selection) == 1)
			{
				gint row_selected = GPOINTER_TO_INT(selection->data);
				if (row_selected != row)
				{
					gtk_clist_unselect_row (clist, row_selected, 0);
					gtk_clist_select_row (clist, row, col);
				}
			}
		}

		search_popup_menu (clist, active_searchlist, row, col, bevent->button, bevent->time);

		return TRUE;	 /* event has been dealt with */
	}

	/* double-click ?
	 * left button double click => download item
	 * middle button double click => download item with custom filename
	 */
	if (event->type == GDK_2BUTTON_PRESS  &&  row >= 0  &&  option_disable_doubleclick == 0)
	{
		GList *original_selection;
		GList *fake_selection;

		/* with multiple selection we only want to download
		 *	the one item double clicked, not all selected items
		 *  => Make fake selection with only one item for
		 *     downloads_popupmenu_item_selected()
		 */
			
		original_selection = selection;
		fake_selection = g_list_append (NULL, GUINT_TO_POINTER(row));
		(active_searchlist->clist)->selection = fake_selection;

		/* middle mouse button? */
		if (bevent->button == 2) 
		{
			search_popupmenu_item_selected (NULL, GUINT_TO_POINTER(SL_MENUITEM_CUSTOM_DL_SELECTED));
		}

		/* or left mouse button? */
		else if (bevent->button == 1)
		{
			search_popupmenu_item_selected (NULL, GUINT_TO_POINTER(SL_MENUITEM_DL_SELECTED));
		} else return FALSE; /* double right click? hmm. ignore */

		if (fake_selection)
			g_list_free (fake_selection);

		(active_searchlist->clist)->selection = original_selection;

		gtk_clist_unselect_all (clist);
		gtk_clist_select_row (clist, row, col);

		return TRUE;	 /* event has been dealt with */
	}

	return FALSE;
}


// search_key_pressed
//
//

static gint
search_key_pressed (GtkWidget *widget, GdkEvent *event)
{
	static guint last_key = 0;

//	printf ("keyval = %u\n", event->key.keyval);
	g_return_val_if_fail (active_searchlist!=NULL, 0);

	if (!active_searchlist->isUserList) // files list
	{
		switch (event->key.keyval){
			case ENTER_KEY:
			{
				if (last_key != RSHIFT_KEY && last_key != LSHIFT_KEY)
				{
					search_popupmenu_item_selected (NULL, GUINT_TO_POINTER(SL_MENUITEM_DL_SELECTED));
					break;
				}
				// no break here, so we go on to custom download
			}
			case 'c':
			case 'C':	search_popupmenu_item_selected (NULL, GUINT_TO_POINTER(SL_MENUITEM_CUSTOM_DL_SELECTED));
						break;
			case 'd':
			case 'D':	search_popupmenu_item_selected (NULL, GUINT_TO_POINTER(SL_MENUITEM_DL_SELECTED));
						break;

			case 'n':
			case 'N':
			case 's':
			case 'S':
			case 'a':
			case 'A':	gtk_widget_grab_focus (ew_addsearch);
						break;
			case 'b':
			case 'B':	search_popupmenu_item_selected (NULL, GUINT_TO_POINTER(SL_MENUITEM_ADD_TO_BLACKLIST));
						break;


			default:		//printf ("keycode = %d\n", event->key.keyval);
						last_key = event->key.keyval;
						return FALSE;
		}
	}
	last_key = event->key.keyval;
	return TRUE; // has been dealt with - if key is not in list we return false right away
}


// Populate search Page  ---------------------------------------------------------------
//
//

// search_notebook_page_switched
//
// Event that occurs when a different page is now the focus
// Note: child of page should be frame
static void
search_notebook_page_switched (GtkWidget *widget, GtkNotebookPage *page, gint page_num)
{
	GtkWidget *child;

	last_time_of_user_click = time(NULL);

	g_return_if_fail (page!=NULL);
	g_return_if_fail (widget!=NULL);
	child = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), page_num);
	g_return_if_fail (child!=NULL);
//	g_return_if_fail (page->child!=NULL);

	active_searchlist = searchlist_find_from_frame (child);
//	active_searchlist = searchlist_find_from_frame (page->child);
	g_return_if_fail (active_searchlist!=NULL);
}


// create_search_notebook
//
// returns search_notebook

static GtkWidget *
create_search_notebook (void)
{
	search_notebook = gtk_notebook_new();
	gtk_widget_show(search_notebook);

	gtk_notebook_set_scrollable (GTK_NOTEBOOK(search_notebook),TRUE);

	// listen for the switch page event
	gtk_signal_connect (GTK_OBJECT (search_notebook), N_("switch_page"),
			  GTK_SIGNAL_FUNC (search_notebook_page_switched), NULL);

	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (search_notebook), option_search_notebook_tabs_pos);	// set position of tabs
	gtk_notebook_set_scrollable (GTK_NOTEBOOK (search_notebook), TRUE);

	searchlist_make_new_searchlist (_("empty"));

	return search_notebook;
}


// searchnotebook_make_new_page
//
// * creates a new search notebook page (=clist),
// * adds it to the search notebook, and
// * sets the given searchList's widget fields

void
searchnotebook_make_new_page (const gchar *tablabel, searchList *sl)
{
	GtkWidget	*hbox, *scrollwin;

	g_return_if_fail (sl!=NULL);

	if (tablabel)
	{
		if (strlen(tablabel)>25)
		{
			gchar shortlabel[27];
			strncpy (shortlabel, tablabel, 25*sizeof(gchar));
			shortlabel[25] = '*';
			shortlabel[26] = 0x00;
			hbox = make_icon_and_label_hbox(shortlabel, icon_search);
		} else hbox = make_icon_and_label_hbox(tablabel, icon_search);
	} else hbox = make_icon_and_label_hbox(N_("???"), icon_search);

	g_return_if_fail (hbox!=NULL);

	if (option_nogreen == 0)
		set_style_recursively (hbox, style_notebook_label);

	create_scrolled_search_clist (&scrollwin, (GtkWidget**) &sl->clist);

 	sl->frame = gtk_frame_new (_(" search results "));
	if (option_nogreen == 0)
	{
		gtk_widget_set_style (sl->frame, style_notebook_page);
		gtk_widget_set_style (search_notebook, style_notebook_page);
	}
	gtk_container_add (GTK_CONTAINER(sl->frame), scrollwin);
	gtk_widget_show(sl->frame);
	gtk_widget_show(hbox);

	gtk_notebook_append_page (GTK_NOTEBOOK (search_notebook), sl->frame, hbox);
	gtk_notebook_set_page (GTK_NOTEBOOK (search_notebook), -1);	// switch to last page (=new page)
}


static void
create_scrolled_search_clist (GtkWidget **p_scrollwin, GtkWidget **p_newclist)
{
	gchar **search_tabtitles = g_strsplit ( _("filename|size in bytes|availability|type|length|bitrate|"
											"codec|artist|title|album|ext|hash"), N_("|"), 0);
	gint i;

	g_return_if_fail (p_scrollwin!=NULL);
	g_return_if_fail (p_newclist!=NULL);

	// creating search clist window and set its properties
	
	*p_newclist = gtk_clist_new_with_titles (NO_OF_SEARCH_COLS, search_tabtitles);

	gtk_signal_connect (GTK_OBJECT (*p_newclist), N_("click-column"),
			    GTK_SIGNAL_FUNC(searchclist_column_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (*p_newclist), N_("resize-column"),
				GTK_SIGNAL_FUNC(searchclist_column_resized), NULL);
	gtk_signal_connect (GTK_OBJECT (*p_newclist), N_("button_press_event"),
				GTK_SIGNAL_FUNC(searchclist_button_pressed), GTK_OBJECT(*p_newclist));
	gtk_signal_connect (GTK_OBJECT (*p_newclist), N_("key_press_event"),
				GTK_SIGNAL_FUNC(search_key_pressed), GTK_OBJECT(*p_newclist));

	gtk_clist_set_sort_column (GTK_CLIST(*p_newclist), searchlist_sortcolumn);
	gtk_clist_set_sort_type (GTK_CLIST(*p_newclist), searchlist_sorttype);
	gtk_clist_set_auto_sort (GTK_CLIST(*p_newclist),FALSE);
	gtk_clist_set_compare_func (GTK_CLIST(*p_newclist), searchclist_compare_func);
	gtk_clist_sort (GTK_CLIST(*p_newclist));

	/* set columns resizable by user (which is not default in gtk2) */
	for (i=0; i<NO_OF_SEARCH_COLS; i++)
	{
		gtk_clist_set_column_resizeable(GTK_CLIST(*p_newclist), i, TRUE);
	}

	// always exactly one item selected (unless less is empty)
	gtk_clist_set_selection_mode(GTK_CLIST (*p_newclist), GTK_SELECTION_EXTENDED);

	// set default column widths
	set_clist_columns (GTK_CLIST(*p_newclist), SEARCH_COLS, NO_OF_SEARCH_COLS);

	gtk_clist_set_column_justification (GTK_CLIST(*p_newclist), 1, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification (GTK_CLIST(*p_newclist), 2, GTK_JUSTIFY_RIGHT);

	// make audio/video cols invisible for now
	for (i=4; i<10; i++)
		gtk_clist_set_column_visibility (GTK_CLIST(*p_newclist), i, FALSE);

	// create a scrollbar window to contain the clist later on

	*p_scrollwin = gtk_scrolled_window_new (NULL,NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(*p_scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	gtk_container_add (GTK_CONTAINER(*p_scrollwin), *p_newclist);
	gtk_widget_show (*p_newclist);
	gtk_widget_show (*p_scrollwin);

	g_strfreev(search_tabtitles);

}

static GtkWidget *
create_searchlist_om_item (GSList **group_p, char *item_text)
{
	GtkWidget *menu_item;

	menu_item = gtk_radio_menu_item_new_with_label (*group_p, item_text);
	*group_p  = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM(menu_item));
	gtk_widget_show (menu_item);

	gtk_signal_connect (GTK_OBJECT (menu_item), N_("activate"),
			    GTK_SIGNAL_FUNC (search_option_menu_item_selected), (gpointer) item_text);

	return (menu_item);
}


static GtkWidget *
create_searchlist_buttons_hbox (void)
{
	GtkWidget	*hbox_search, *menu_search, *vbox, *hbox2, *hbox21, *hbox22;
	GtkWidget	*label1, *label2, *label3, *label4, *label6, *label7, *label8;
	GtkWidget	*reset_button, *searchoptshow_button; 
	GtkTooltips *tooltips;
	GSList		*group;	/* for option menu */

	tooltips = gtk_tooltips_new();

	/* a vbox containing two rows of hboxes, viz. hbox2 on top, and hbox_search at the bottom */
	vbox = gtk_vbox_new (TRUE,5);
	gtk_widget_show (vbox);

	/* hbox2 is the whole upper row */
	hbox2 = gtk_hbox_new (FALSE,5);
	gtk_box_pack_start (GTK_BOX(vbox), hbox2, TRUE, TRUE, 0);
	search_hbox_extra_options = hbox2;

	/* hbox_search is the whole lower row */
	hbox_search = gtk_hbox_new (FALSE,5);
	gtk_widget_show (hbox_search);
	gtk_box_pack_start (GTK_BOX(vbox), hbox_search, TRUE, TRUE, 0);

	/* hbox21 is the left half of the upper row */
	hbox21 = gtk_hbox_new(FALSE,0);
	gtk_widget_show (hbox21);
	gtk_box_pack_start (GTK_BOX(hbox2), hbox21, TRUE, TRUE, 0);

	/* hbox22 is the right half of the upper row */
	hbox22 = gtk_hbox_new(FALSE,0);
	gtk_widget_show (hbox22);
	gtk_box_pack_start (GTK_BOX(hbox2), hbox22, TRUE, TRUE, 0);

	/******************************************************
	 * fill upper hbox with advanced search options
	 ******************************************************/

	label1 = gtk_label_new (_(" size between "));
	gtk_widget_show(label1);
	gtk_box_pack_start (GTK_BOX(hbox21), label1, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL(label1), GTK_JUSTIFY_RIGHT);
	if (option_nogreen == 0)
		gtk_widget_set_style (label1, style_notebook_label);

	minsize = gtk_spin_button_new (GTK_ADJUSTMENT(gtk_adjustment_new(0.0,0.0,4096.0,1.0,50.0,50.0)), 1.0, 0);
	gtk_widget_show (minsize);
	gtk_box_pack_start (GTK_BOX(hbox21), minsize, TRUE, TRUE, 0);

	label2 = gtk_label_new (_(" and "));
	gtk_widget_show(label2);
	gtk_box_pack_start (GTK_BOX(hbox21), label2, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL(label2), GTK_JUSTIFY_CENTER);
	if (option_nogreen == 0)
		gtk_widget_set_style (label2, style_notebook_label);

	maxsize = gtk_spin_button_new (GTK_ADJUSTMENT(gtk_adjustment_new(4096.0,0.0,4096.0,1.0,50.0,50.0)), 1.0, 0);
	gtk_widget_show (maxsize);
	gtk_box_pack_start (GTK_BOX(hbox21), maxsize, TRUE, TRUE, 0);

	label3 = gtk_label_new (_(" MB"));
	gtk_widget_show(label3);
	gtk_box_pack_start (GTK_BOX(hbox21), label3, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL(label3), GTK_JUSTIFY_LEFT);
	if (option_nogreen == 0)
		gtk_widget_set_style (label3, style_notebook_label);

	label6 = gtk_label_new (N_("  "));
	gtk_widget_show(label6);
	gtk_box_pack_start (GTK_BOX(hbox22), label6, TRUE, TRUE, 0);

	label7 = gtk_label_new (_("bitrate >= "));
	gtk_widget_show(label7);
	gtk_box_pack_start (GTK_BOX(hbox22), label7, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL(label7), GTK_JUSTIFY_RIGHT);
	if (option_nogreen == 0)
		gtk_widget_set_style (label7, style_notebook_label);

	minbitrate = gtk_spin_button_new (GTK_ADJUSTMENT(gtk_adjustment_new(128.0, 32.0,512.0, 16.0,32.0,32.0)), 16.0, 0);
	gtk_widget_show (minbitrate);
	gtk_widget_set_sensitive (minbitrate, FALSE);
	gtk_box_pack_start (GTK_BOX(hbox22), minbitrate, TRUE, TRUE, 0);

	label8 = gtk_label_new (N_("  "));
	gtk_widget_show(label8);
	gtk_box_pack_start (GTK_BOX(hbox22), label8, TRUE, TRUE, 0);

	label4 = gtk_label_new (_(" min. avail. "));
	gtk_label_set_justify (GTK_LABEL(label4), GTK_JUSTIFY_RIGHT);
	gtk_widget_show(label4);
	gtk_box_pack_start (GTK_BOX(hbox22), label4, FALSE, FALSE, 0);
	if (option_nogreen == 0)
		gtk_widget_set_style (label4, style_notebook_label);

	minavail = gtk_spin_button_new (GTK_ADJUSTMENT(gtk_adjustment_new(1.0,1.0,9999.0,1.0,10.0,10.0)), 1.0, 0);
	gtk_widget_show (minavail);
	gtk_box_pack_start (GTK_BOX(hbox22), minavail, TRUE, TRUE, 0);

/*	label5 = gtk_label_new (N_("   "));
	gtk_widget_show(label5);
	gtk_box_pack_start (GTK_BOX(hbox22), label5, TRUE, TRUE, 0);
*/

	/**************************************
	 * lower row of entry widget + buttons
	 **************************************/

	// extra search options show/hide toggle button
	new_icon_toggle_button_pack_and_signal_connect (&searchoptshow_button, hbox_search, N_(""),
					    search_popupmenu_item_selected, GUINT_TO_POINTER(SL_MENUITEM_TOGGLE_SEARCHOPT), 1, icon_options);
	gtk_tooltips_set_tip(tooltips, searchoptshow_button, _("show/hide extra search options"), NULL);

	// reset values button
	new_icon_button_pack_and_signal_connect (&reset_button, hbox_search, N_(""),
					    search_popupmenu_item_selected, GUINT_TO_POINTER(SL_MENUITEM_RESET), 1, icon_refresh);
	gtk_tooltips_set_tip(tooltips, reset_button, _("set all search options back to their default values"), NULL);

	/* create search types options menu */

	om_search   = gtk_option_menu_new();		/* create the option menu */

	menu_search = gtk_menu_new();			/* create a menu          */
	group	    = NULL;

	gtk_menu_append (GTK_MENU(menu_search), create_searchlist_om_item (&group, _("Any")));
	gtk_menu_append (GTK_MENU(menu_search), create_searchlist_om_item (&group, _("Audio")));
	gtk_menu_append (GTK_MENU(menu_search), create_searchlist_om_item (&group, _("Video")));
//	gtk_menu_append (GTK_MENU(menu_search), create_searchlist_om_item (&group, _("User")));
	gtk_menu_append (GTK_MENU(menu_search), create_searchlist_om_item (&group, _("Images")));
	gtk_menu_append (GTK_MENU(menu_search), create_searchlist_om_item (&group, _("Programs")));
	gtk_menu_append (GTK_MENU(menu_search), create_searchlist_om_item (&group, _("Documents")));
	gtk_widget_show (menu_search);

	last_searchtype_selected = g_string_new (_("Any"));

	gtk_option_menu_set_menu (GTK_OPTION_MENU(om_search), menu_search);
	gtk_widget_show (om_search);
	gtk_box_pack_start (GTK_BOX (hbox_search), om_search, FALSE, TRUE,2);

	// create text entry widget
		ew_addsearch = gtk_entry_new_with_max_length (100);
	gtk_entry_set_text (GTK_ENTRY (ew_addsearch), DEFAULT_SEARCH_PHRASE);
	gtk_signal_connect (GTK_OBJECT (ew_addsearch), N_("changed"),
			    GTK_SIGNAL_FUNC (search_entry_widget_changed), NULL);
	gtk_signal_connect (GTK_OBJECT (ew_addsearch), N_("activate"),
			    GTK_SIGNAL_FUNC (search_add_search_button_clicked),
			    ew_addsearch);
	/* make num block enter key behave like normal enter key */
	gtk_signal_connect (GTK_OBJECT (ew_addsearch), N_("insert_text"),
				GTK_SIGNAL_FUNC (misc_gtk_entry_insert_text_handler), ew_addsearch);

	gtk_box_pack_start (GTK_BOX (hbox_search), ew_addsearch, FALSE, TRUE,2);
	gtk_widget_show (ew_addsearch);

	new_icon_button_pack_and_signal_connect (&button_addsearch, hbox_search , N_(""),
					    search_add_search_button_clicked, ew_addsearch, 1, icon_search);
	gtk_tooltips_set_tip(tooltips, button_addsearch, _("search your main server"), NULL);

	// extend search button

	new_icon_button_pack_and_signal_connect (&button_extend, hbox_search , _(" extend 5x "),
					    search_popupmenu_item_selected, GUINT_TO_POINTER(SL_MENUITEM_EXTEND_SEARCH_5X), 1, icon_extend);
 	gtk_widget_set_sensitive (button_extend, FALSE);
	gtk_tooltips_set_tip(tooltips, button_extend, _("search five other servers (note: you'll need to search your main server first!)"), NULL);

	// more search button

	new_icon_button_pack_and_signal_connect (&button_more, hbox_search , _(" more "),
					    search_popupmenu_item_selected, GUINT_TO_POINTER(SL_MENUITEM_GET_NEXT_200), 1, icon_200more);
 	gtk_widget_set_sensitive (button_more, FALSE);
	gtk_tooltips_set_tip(tooltips, button_more, _("get the next 200 matches from the main server (if there are more matches)"), NULL);

	// 'forget search' search button 
	new_icon_button_pack_and_signal_connect (&button_forget, hbox_search , _(" forget "),
					    search_popupmenu_item_selected, GUINT_TO_POINTER(SL_MENUITEM_FORGET_SEARCH), 1, icon_cancel);
	gtk_tooltips_set_tip(tooltips, button_forget, _("clear and remove the current search results page"), NULL);

	return (vbox);
}


/* search_page_create
 *
 *
 *
 */

GtkWidget *
search_page_create (void)
{

	GtkWidget	*table_search;

	table_search = gtk_table_new (1,2,FALSE);	// 1x2, not homogeneous
	gtk_widget_show (table_search);

	// create scrolled search clist and add it to search table

	gtk_table_attach (GTK_TABLE (table_search),
				create_search_notebook(),
			  0,1,0,1, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
			  	   GTK_FILL | GTK_SHRINK | GTK_EXPAND, 5,5);

	// create horizontal packing box for search control buttons and add it to table

	gtk_table_attach (GTK_TABLE(table_search),
			  create_searchlist_buttons_hbox(),
			  0,1,1,2, GTK_SHRINK | GTK_EXPAND,
			           GTK_SHRINK | GTK_FILL, 50,5);

	return table_search;

/*	GtkWidget *vbox;

	vbox = gtk_vbox_new (FALSE, 5);
	g_return_if_fail (vbox!=NULL);

	gtk_box_pack_start (GTK_BOX(vbox), create_search_notebook(), TRUE, TRUE, 5);
	gtk_box_pack_end (GTK_BOX(vbox), create_searchlist_buttons_hbox(), FALSE, FALSE, 5);

	// add search page vox to frame and make visible
	gtk_container_add (GTK_CONTAINER (frame), vbox);
	gtk_widget_show (vbox);
*/
}


void
search_page_set_initial_extra_search_options_visibility (void)
{
	if (option_show_extra_search_options == 1)
	{
//		g_print ("\n\nshow\n\n");
		gtk_widget_show (search_hbox_extra_options);
	} else { gtk_widget_hide (search_hbox_extra_options); 	/*	g_print ("\n\nhide\n\n");*/ }
}


/******************************************************************************
 *
 *   searchclist_resort
 *
 *
 */

void
searchclist_resort (GtkCList *clist)
{
	g_return_if_fail (clist!=NULL);

	gtk_clist_set_auto_sort (clist, FALSE);
	gtk_clist_set_compare_func (clist, searchclist_compare_func);
	gtk_clist_sort (clist);
}



