/***************************************************************************
       shared_files.c  -  take care of all the shared files stuff (clist)
                             -------------------
    begin                : Wed Jul 10 2002
    copyright            : (C) 2001-2002 by Tim-Philipp Mller
    email                : t.i.m at orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "gui_options.h"
#include "icons.h"
#include "misc.h"
#include "shared_files.h"
#include "status_page.h"
#include "uploads_page.h"

#include <stdlib.h>
#include <stdio.h>

#ifndef SF_CLIST
# define SF_CLIST	GTK_CLIST(clist_files)
#endif


GtkWidget		*clist_files = NULL;		// shared files clist widget

GPtrArray		*sharedfiles = NULL;		// shared files data


static void	shared_files_clist_resort (void);
static void	shared_files_clist_column_clicked (GtkCList *clist, gint column, gpointer data);
static gint	shared_files_clist_compare_func (GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2);
static gint	shared_files_button_pressed (GtkWidget *widget, GdkEvent *event);
static void	shared_files_popup_menu (guint row, guint col, guint button, guint32 activate_time);

static void	shared_files_remove_record (sharedfileRecord *sfr);
static void	shared_files_free_record (sharedfileRecord *sfr);
static void	shared_files_clist_add_or_update_record (sharedfileRecord *sfr);



// shared_files_clist_clear
//
// clears the shared files clist

void
shared_files_clist_clear(void)
{
	if (sharedfiles)
	{
		PROCESS_GPTRARRAY_FROM_END( sharedfiles, sharedfileRecord,
			if (item) shared_files_remove_record (item);
		);
	}
	gtk_clist_clear(SF_CLIST);
}


// shared_files_find_from_filename
//
// Find a record in the shared files list from the filename
// Returns record or NULL if not found

sharedfileRecord *
shared_files_find_from_filename (const gchar *name)
{
	g_return_val_if_fail (name!=NULL, NULL);

	if (sharedfiles)
	{
		GQuark namequark = g_quark_try_string (name);
		if (namequark>0)
		{
			PROCESS_GPTRARRAY( sharedfiles, sharedfileRecord,
				if ((item) && item->namequark == namequark) return item;
			);
		}
	}
	return NULL;
}


// shared_files_find_from_filenamequark
//
// Find a record in the shared files list from the filename quark
// Returns record or NULL if not found

sharedfileRecord *
shared_files_find_from_filenamequark (GQuark namequark)
{
	if ((sharedfiles) && namequark>0)
	{
		PROCESS_GPTRARRAY( sharedfiles, sharedfileRecord,
			if ((item) && item->namequark == namequark) return item;
		);
	}
	return NULL;
}



// shared_files_find_from_hash
//
// Find a record in the shared files list from the filehash.
// Returns record or NULL if not found

sharedfileRecord *
shared_files_find_from_hash (const guint8 *hash)
{
	g_return_val_if_fail (hash!=NULL, NULL);
	if (sharedfiles)
	{
		PROCESS_GPTRARRAY( sharedfiles, sharedfileRecord,
			if ((item) && memcmp(item->hash, hash, 16)==0)
				return item;
		);
	}
	return NULL;
}



// shared_files_clist_add_or_update_record
//
// adds or updates a record to/in the shared files clist

static void
shared_files_clist_add_or_update_record (sharedfileRecord *sfr)
{
	gchar	*cols[4];
	gint	 row;

	g_return_if_fail(sfr!=NULL);

	cols[0] = g_quark_to_string(sfr->namequark);
	cols[1] = (gchar*) human_size(sfr->filesize);
	cols[2] = g_quark_to_string(sfr->formatquark);
	cols[3] = g_quark_to_string(sfr->typequark);

	row = gtk_clist_find_row_from_data (SF_CLIST, (gpointer) sfr);
	if (row>=0)
	{
		if (cols[0]) gtk_clist_set_text (SF_CLIST, row, 0, cols[0]);
		if (cols[1]) gtk_clist_set_text (SF_CLIST, row, 1, cols[1]);
		if (cols[2]) gtk_clist_set_text (SF_CLIST, row, 2, cols[2]);
		if (cols[3]) gtk_clist_set_text (SF_CLIST, row, 3, cols[3]);
	} else row = gtk_clist_append (SF_CLIST, cols);
	gtk_clist_set_row_data (SF_CLIST, row, (gpointer) sfr);
}


// shared_files_clist_add_or_update_record
//
// Adds a record to the shared files list and clist. If it already exists,
//	the fields are updated.

void
shared_files_add_or_update_record (	const guint8 *hash,
					const gchar *name,
					guint32 size,
					const gchar *format,
					const gchar *type	)
{
	sharedfileRecord	*sfr;
	GQuark			 namequark;
	gboolean		 update_clist = FALSE;

	g_return_if_fail (hash!=NULL);
	g_return_if_fail (name!=NULL);

	sfr = shared_files_find_from_hash (hash);

	// if record exists, just update name (quark)
	if (!sfr)
	{
		sfr = g_new0 (sharedfileRecord, 1);
		g_return_if_fail (sfr!=NULL);
		memcpy (sfr->hash, hash, 16);
		sfr->filesize = size;
		sfr->formatquark = (format) ? g_quark_from_string(format) : 0;
		sfr->typequark   = (type)   ? g_quark_from_string(type) : 0;

		// add to list [pointer array]
		if (!sharedfiles) sharedfiles = g_ptr_array_new();
		g_ptr_array_add (sharedfiles, (gpointer) sfr);
	
		update_clist = TRUE;		// add to clist
	}

	namequark = g_quark_from_string (name);
	if (sfr->namequark!=namequark || (update_clist))
	{
		sfr->namequark = namequark;
		shared_files_clist_add_or_update_record (sfr);
	}

	sfr->has_not_been_updated = FALSE;
}

static void
shared_files_free_record (sharedfileRecord *sfr)
{
	g_return_if_fail (sfr!=NULL);
	memset(sfr,0x00,sizeof(sfr));
	g_free(sfr);
}


// shared_files_remove_record
//
// removes a record from the shared files list and clist
//	and frees its memory

static void
shared_files_remove_record (sharedfileRecord *sfr)
{
	gint row;
	g_return_if_fail (sfr!=NULL);
	g_return_if_fail (sharedfiles!=NULL);
	g_ptr_array_remove_fast (sharedfiles, (gpointer) sfr);
	shared_files_free_record (sfr);

	// now remove from clist as well
	row = gtk_clist_find_row_from_data (SF_CLIST, (gpointer)sfr);
	if (row>=0) gtk_clist_remove (SF_CLIST, row);
}

// shared_files_clist_before_update
// shared_files_clist_after_update
//
// These two functions are called before and after the clist is updated on a large
//	scale (from the parse message function). (see top of file why we do that)

void
shared_files_clist_before_update (void)
{
	if (sharedfiles)
	{
		PROCESS_GPTRARRAY( sharedfiles, sharedfileRecord,
			if (item) item->has_not_been_updated = TRUE;
		);
		gtk_clist_freeze (SF_CLIST);
	}
}

void
shared_files_clist_after_update (void)
{
	shared_files_clist_resort ();

	if (sharedfiles)
	{
		PROCESS_GPTRARRAY_FROM_END( sharedfiles, sharedfileRecord,
			if ((item) && (item->has_not_been_updated))
				shared_files_remove_record (item);
		);
		gtk_clist_thaw (SF_CLIST);
	}
}


// shared_files_clist_resort
//
// resort the clist

static void
shared_files_clist_resort (void)
{
	gtk_clist_set_auto_sort (SF_CLIST, FALSE);
	gtk_clist_set_compare_func (SF_CLIST, shared_files_clist_compare_func);
	gtk_clist_set_sort_column (SF_CLIST, sharedfileslist_sortcolumn);
	gtk_clist_set_sort_type (SF_CLIST, sharedfileslist_sorttype);
	gtk_clist_sort (SF_CLIST);
}


// shared_files_clist_column_clicked
//
// One of the column headers has been clicked => change list sorting

static void
shared_files_clist_column_clicked (GtkCList *clist, gint column, gpointer data)
{
	// column clicked is already active for sorting? -> change sort type
	if (column == sharedfileslist_sortcolumn)
	{
		if (sharedfileslist_sorttype == GTK_SORT_ASCENDING)
			sharedfileslist_sorttype = GTK_SORT_DESCENDING;
		else sharedfileslist_sorttype = GTK_SORT_ASCENDING;
	} else sharedfileslist_sorttype   = GTK_SORT_ASCENDING;
		
	sharedfileslist_sortcolumn = column;
	shared_files_clist_resort ();
}



// shared_files_clist_compare_func
//
// compares two rows from the shared files clist

static gint
shared_files_clist_compare_func (GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
	gchar	*text1=NULL, *text2=NULL;
	gint	 ret;

	g_return_val_if_fail (clist!=NULL, 0);

	get_text_from_gtk_clist_rows ((GtkCListRow *) ptr1, (GtkCListRow *) ptr2, clist->sort_column, &text1, &text2);

	if (!text2) return (text1 != NULL);
	if (!text1) return -1;

	// sort depending on type of content (i.e. which column is the one to sort)
	// only one column currently...
	switch (clist->sort_column)	// XXX - direct access of structure not kosher
	{
		case 1:		ret = compare_file_size_str (text1,text2);
		break;

		default:	ret = g_strcasecmp (text1, text2);
		break;
	}
	// If two rows are equal and we sort by something different than the filename,
	//	then use filename as secondary sort criterion
	if (ret==0 && clist->sort_column!=0)
	{
		// Get filenames ...
		get_text_from_gtk_clist_rows ((GtkCListRow *) ptr1, (GtkCListRow *) ptr2, 0, &text1, &text2);
		// .. and compare them
		ret = g_strcasecmp (text1, text2);
		// If the other column is sorted in descending order, do still sort
		//	by filenames in ascending order:
		if (sharedfileslist_sorttype == GTK_SORT_DESCENDING)
			ret = (ret < 0) ? ret = 1 : (ret>0) ? ret = -1 : 0;
	}
	return ret;
}


// shared_files_column_resized
//
// A column of the shared files list has been resized. Remember new value
//	so we can save it in the options file

static void
shared_files_column_resized (GtkCList *widget, gint col, gint new_width, gpointer data)
{
	column_widths[FSHARED_COLS][col] = new_width;
}



// shared_files_popupmenu_item_selected
//
// An item from the shared files pop-up menu has been selected. Take some action...
//
// data contains the menuitem selected (lowest 8 bits)
// Note: widget might be NULL

static void
shared_files_popupmenu_item_selected (GtkWidget *widget, gpointer data)
{
	guint		menuitem =  GPOINTER_TO_UINT (data) & 0xff;

	switch (menuitem)
	{
		case SF_MENUITEM_LINK_TO_LOG:
		{
			PROCESS_CLIST_SELECTION_LIST (SF_CLIST, sharedfiles, sharedfileRecord,
				if (item)
				{
					gchar *name = g_quark_to_string (item->namequark);
					if (name) status_msg_print_ed2klink (name, item->filesize, item->hash);
				}
			);
		}
		break;

		case SF_MENUITEM_CLEAR_SELECTION:
		{
			gtk_clist_unselect_all (SF_CLIST);
		}
		break;

		case SF_MENUITEM_SELECT_ALL:
		{
			gtk_clist_select_all (SF_CLIST);
		}
		break;

		default:
			g_printerr (_("GUI: Fix me - called with unknown value %u (%s)\n"), menuitem, __FUNCTION__);
		break;
	}
}


// shared_files_popup_menu
//
// Creates a pop-up menu for the shared files list and pops it up
// row, col: clist row and column where click took place

static void
shared_files_popup_menu (guint row, guint col, guint button, guint32 activate_time)
{
#	define SF_ITEMS_INSENSITIVE_ON_NO_SELECTION	2
	// menu widget isn't destroyed automatically => we have to destroy it after use
	static GtkWidget	*menu = NULL;
	GtkWidget		*mi[SF_ITEMS_INSENSITIVE_ON_NO_SELECTION];

	const gchar	*SELECT_ALL_STR =	_(" select all ");
	const gchar	*CLEAR_SELECT_STR =	_(" unselect all ");
	const gchar	*LOG_LINK_STR =		_(" write ed2k-link to log/clipboard ");

	if (menu)	// destroy (and thus free) last pop-up menu
	{
		gtk_widget_destroy(menu);
		menu=NULL;
	}

	menu  = gtk_menu_new();

//	if (option_DLSingleSelectionMode==0)
//	{
		(void)	MENU_ITEM_ADD (SELECT_ALL_STR, shared_files_popupmenu_item_selected, SF_MENUITEM_SELECT_ALL, icon_none);
//	}
	mi[0] = MENU_ITEM_ADD (CLEAR_SELECT_STR, shared_files_popupmenu_item_selected, SF_MENUITEM_CLEAR_SELECTION, icon_none);
	(void)	MENU_ITEM_ADD__________;
	mi[1] = MENU_ITEM_ADD (LOG_LINK_STR, shared_files_popupmenu_item_selected, SF_MENUITEM_LINK_TO_LOG, icon_status);

	// No items selected? => make those functions that need items selected insensitive
	if (!SF_CLIST->selection)
	{
		guint i;
		for (i=0; i<SF_ITEMS_INSENSITIVE_ON_NO_SELECTION; i++)
			gtk_widget_set_sensitive (GTK_WIDGET(mi[i]), FALSE);
	}

	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, activate_time);
}


// shared_files_button_pressed
//
// Shows the shared files pop-up menu on right-click on the clist
// writes ed2k-link to status window on double-click on row (if double-click not disabled in options)
//
// This right-click pop-up menu code was inspired by code posted by Mike Hicks to
//	the gtk-list@redhat.com mailing list on 25/11/98. Thank you, Mike :)

static gint
shared_files_button_pressed (GtkWidget *widget, GdkEvent *event)
{
	guint	row, col;

//	last_time_of_user_click = time(NULL);

	if (event->type == GDK_BUTTON_PRESS)
	{
		// convert GdkEvent to GdkEventButton
		GdkEventButton *bevent = (GdkEventButton *) event;
		if (bevent->button == 3)	// right-click
		{
			gtk_clist_get_selection_info (SF_CLIST, (int) bevent->x ,(int) bevent->y, &row, &col);
			// if nothing selected, select row of click
			if ((SF_CLIST) && (!SF_CLIST->selection))
				gtk_clist_select_row (SF_CLIST, row, col);
			shared_files_popup_menu (row, col, bevent->button, bevent->time);

		}
		return TRUE;	 // event has been dealt with
	}

	// double-click? => toggle pause/resume download
	if ((event->type) == GDK_2BUTTON_PRESS  && option_disable_doubleclick==0)
	{
		// convert GdkEvent to GdkEventButton
		GdkEventButton *bevent = (GdkEventButton *) event;
		g_return_val_if_fail(SF_CLIST!=NULL,FALSE);
		gtk_clist_get_selection_info (SF_CLIST, (int) bevent->x ,(int) bevent->y, &row, &col);
		if (row < SF_CLIST->rows && row>=0)	// otherwise click on empty space
		{
			GList		*original_selection, *fake_selection;
			original_selection = SF_CLIST->selection;
			// Make fake selection with only one item for shared_files_popupmenu_item_selected()
			fake_selection = g_list_append (NULL, GUINT_TO_POINTER(row));
			SF_CLIST->selection = fake_selection;
			shared_files_popupmenu_item_selected (NULL, GUINT_TO_POINTER(SF_MENUITEM_LINK_TO_LOG));
			if (fake_selection) g_list_free (fake_selection);
			SF_CLIST->selection = original_selection;
		}
		return TRUE;
	}

	return FALSE;
}


// create_scrolled_files_shared_clist
//
// Creates the shared files clist embedded in a scrollwin.
// Returns the scrollwin widget.

GtkWidget *
shared_files_create_scrolled_clist (void)
{
	GtkWidget	 *scrollwin;
	gchar		**tabtitles = g_strsplit ( _(" files shared - filename|size | format| type"), N_("|"), 0);
	gint		  i;

	g_return_val_if_fail (tabtitles!=NULL,NULL);

	// creating shared clist window and set its properties

	clist_files = gtk_clist_new_with_titles (4, tabtitles);	// 4 columns
	gtk_signal_connect (GTK_OBJECT (clist_files), N_("click-column"),
			    GTK_SIGNAL_FUNC(shared_files_clist_column_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_files), N_("resize-column"),
				GTK_SIGNAL_FUNC(shared_files_column_resized), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_files), N_("button_press_event"),
				GTK_SIGNAL_FUNC(shared_files_button_pressed), NULL);

	shared_files_clist_resort();

	/* set columns resizable by user (which is not default in gtk2) */
	for (i=0; i<4; i++)
	{
		gtk_clist_set_column_resizeable(GTK_CLIST(clist_files), i, TRUE);
	}

	// justify file size column right
	gtk_clist_set_column_justification (SF_CLIST, 1, GTK_JUSTIFY_RIGHT);

	gtk_clist_set_selection_mode(SF_CLIST, GTK_SELECTION_EXTENDED);
	gtk_clist_set_shadow_type(SF_CLIST, GTK_SHADOW_NONE);	// define look
	set_clist_columns (SF_CLIST, FSHARED_COLS, 4);		// set default column widths

	// create a scrollbar window to contain the clist
	scrollwin = gtk_scrolled_window_new (NULL,NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER(scrollwin), clist_files);
	gtk_widget_show (clist_files);
	gtk_widget_show (scrollwin);

	g_strfreev(tabtitles);

	return (scrollwin);
}
