/***************************************************************************
                          tag.c  -  description
                             -------------------
    begin                : Sat May 11 2002
    copyright            : (C) 2002 by Tim-Philipp Mller
    email                : t.i.m at orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "tag.h"
#include "ramfile.h"
#include <string.h>	// for memset, strlen

// special tags: abbreviations

// tag_new
//
// create a new tag with name and type
//
// returns pointer to new tag or NULL on error

inline NGS_tag *
tag_new (guint8 type, const gchar *name)
{
	NGS_tag	*newtag = g_new0 (NGS_tag,1);
	g_return_val_if_fail (newtag!=NULL, NULL);	// mem alloc problem
	g_return_val_if_fail (type>TAGTYPE_UNDEFINED && type<TAGTYPE_LAST_UNDEFINED, NULL);	// tagtype invalid
	newtag->type = type;
	newtag->name = g_strdup(name);
	g_return_val_if_fail (newtag->name!=NULL, NULL); // mem alloc problem
	newtag->data = NULL;
	return newtag;
}

// tag_new_hash
//
// create a new hash tag with name
//
// returns pointer to new tag or NULL on error

NGS_tag *
tag_new_hash (const gchar *name, const guint8 *hash)
{
	NGS_tag *nt;
	g_return_val_if_fail (hash!=NULL, NULL);
	nt = tag_new(TAGTYPE_HASH, name);
	if (nt)	// if there was an error, tag_new will have generated an error message
	{
		nt->data = g_memdup (hash,16);
		g_return_val_if_fail (nt->data!=NULL, NULL);	// mem alloc problem
	}
	return nt;
}

// tag_new_integer
//
// create a new integer tag with name
//
// returns pointer to new tag or NULL on error

NGS_tag *
tag_new_integer (const gchar *name, gint32 val)
{
	NGS_tag *nt = tag_new(TAGTYPE_INT, name);
	if (nt)	// if there was an error, tag_new will have generated an error message
	{
		nt->data = GUINT_TO_POINTER(val);
	}
	return nt;
}

// tag_new_string
//
// create a new string tag with name
//
// returns pointer to new tag or NULL on error

NGS_tag *
tag_new_string (const gchar *name, const gchar *stringval)
{
	NGS_tag *nt = tag_new(TAGTYPE_STRING, name);
	if (nt)	// if there was an error, tag_new will have generated an error message
	{
		nt->data = (gpointer) g_strdup(stringval);
		g_return_val_if_fail (nt->data!=NULL, NULL);	// mem alloc problem
	}
	return nt;
}

// tag_new_float
//
// create a new float tag with name
//
// returns pointer to new tag or NULL on error

NGS_tag *
tag_new_float (const gchar *name, gfloat fval)
{
	NGS_tag *nt = tag_new(TAGTYPE_FLOAT, name);
	if (nt)	// if there was an error, tag_new will have generated an error message
	{
		nt->data = *((gpointer*)&fval);
	}
	return nt;
}

// tag_new_from_ramfile
//
// creates a tag from ramfile stream
//
// returns pointer to new tag or NULL on error

NGS_tag *
tag_new_from_ramfile (ramfile *rf)
{
	NGS_tag	*nt = NULL;

	nt = g_new0 (NGS_tag,1);
	g_return_val_if_fail (nt!=NULL, NULL);	// mem alloc problem
	nt->type = ramfile_read8(rf);
	nt->name = ramfile_read_i4_string(rf);
	switch (nt->type)
	{
		case TAGTYPE_HASH:
		{
			nt->data = g_new0 (guint8,16);
			g_return_val_if_fail (nt->data!=NULL, NULL);	// mem alloc problem
			ramfile_read(rf,nt->data,16);
		}
		break;

		case TAGTYPE_STRING:	nt->data = (gpointer) ramfile_read_i4_string(rf);
		break;
		case TAGTYPE_INT:	nt->data = GUINT_TO_POINTER(ramfile_read32(rf));
		break;

		case TAGTYPE_FLOAT:
		{
//			gfloat	fval;
			nt->data = g_new(gfloat,1);
			g_return_val_if_fail (nt->data!=NULL, NULL);	// mem alloc problem
#warning this will probably only work on x86 systems!
			ramfile_read (rf, (guint8*)nt->data, 4);
		}
		break;

		case TAGTYPE_BOOL:
		case TAGTYPE_BOOLARRAY:
		case TAGTYPE_BLOB:
		default:
		{
			g_printerr (_("tag_new_from_ramfile: unknown or unimplemented tag type %u!\n"), nt->type);
//			g_free(name);
			return NULL;
		}
		break;
	}
//	g_free(name);

	return nt;
}

// tag_free
//
// frees tag content and structure

void
tag_free (NGS_tag *tag)
{
	tag_free_content (tag);
	if (tag) g_free (tag);
}

// tag_free_content
//
// frees tag's content only, but not the structure (necessary if tag not dyn.allocated)

void
tag_free_content (NGS_tag *tag)
{
	g_return_if_fail (tag!=NULL);
	if (tag->name) g_free (tag->name);
	switch (tag->type)
	{
		case TAGTYPE_HASH:
		{
			if (tag->data) g_free ((guint8*)tag->data);
		}
		break;

		case TAGTYPE_STRING:
		{
			if (tag->data) g_free ((gchar*)tag->data);
		}
		break;

		case TAGTYPE_INT:
		break;			// nothing to do

		case TAGTYPE_FLOAT:
		{
			if (tag->data) g_free(tag->data);	
		}
		break;

		case TAGTYPE_BOOL:
		case TAGTYPE_BOOLARRAY:
		case TAGTYPE_BLOB:
		default:
		{
			g_printerr (_("tag_free_content: unknown or unimplemented tag type!\n"));
		}
	}
}


void
tag_dump (NGS_tag *tag)
{
	g_return_if_fail (tag!=NULL);
	g_return_if_fail (tag->name!=NULL);
//	g_print ("----- tag dump: -----\n\t* name = '%s'\n", tag_get_name(tag));
	g_print (N_("\t%s\t = "), tag_get_name(tag));	
	switch (tag->type)
	{
		case TAGTYPE_HASH:
		{
			g_print (_("hash dump not implemented\n"));
		}
		break;

		case TAGTYPE_STRING:
		{
			g_print (N_("'%s'\n"), (gchar*)tag->data);
		}
		break;

		case TAGTYPE_INT:
		{
			g_print (N_("%u\n"), GPOINTER_TO_UINT(tag->data));
		}
		break;

		case TAGTYPE_FLOAT:
		{
			g_print (N_("%f\n"), *((gfloat*)tag->data));
		}
		break;

		case TAGTYPE_BOOL:
		case TAGTYPE_BOOLARRAY:
		case TAGTYPE_BLOB:
		default:
		{
			g_print (_("unknown or unimplemented tag type!\n"));
		}
	}
}

void
tag_write_to_ramfile (NGS_tag *tag, ramfile *rf)
{
	g_return_if_fail (tag!=NULL);
	g_return_if_fail (rf!=NULL);

	ramfile_write8(rf, tag->type);
	ramfile_write_i4_string (rf, ((tag->name)) ? tag->name : "");

	switch (tag->type)
	{
		case TAGTYPE_HASH:
		{
			if (!tag->data)
			{
				guint8	emptybuf[16];
				memset (emptybuf, 0x00, 16);
				ramfile_write (rf, emptybuf, 16);
			} else ramfile_write (rf, (guint8*)tag->data, 16);
		}
		break;

		case TAGTYPE_STRING:
		{
			gchar* gstr = (gchar*) tag->data;
			ramfile_write_i4_string (rf, ((gstr)) ? gstr : "");
		}
		break;

		case TAGTYPE_INT:
		{
			ramfile_write32 (rf, GPOINTER_TO_UINT(tag->data));
		}
		break;

		case TAGTYPE_FLOAT:
		{
			g_print (_("%s: write float tag - IMPLEMENT ME!\n"), __FUNCTION__);
			ramfile_write32 (rf, 0);	// just write something for now
		}
		break;

		case TAGTYPE_BOOL:
		case TAGTYPE_BOOLARRAY:
		case TAGTYPE_BLOB:
		default:
		{
			g_printerr (_("\t* val = unknown or unimplemented tag type!\n"));
		}
	}
}

/* this is only for debugging purposes really and doesn't need translating: */

static const gchar *TAGNAMES_SHORT_CLEARTEXT[] = {
N_("undefined(0)"),
N_("file name"),
N_("file size"),
N_("file type"),
N_("file format"),
N_("not implemented yet (5)"),
N_("not implemented yet (6)"),
N_("not implemented yet (7)"),
N_("amount transfered so far"),
N_("gap start"),
N_("gap end"),
N_("server description"),
N_("server ping"),
N_("server connection history"),
N_("server preference"),
N_("server port"),
N_("server IP"),
N_("user's client version (=>hex)"),
N_("temp file name"),
N_("priority"),
N_("state"),
N_("availability"),
N_("qtime (last time client asked to be on Q)"),
N_("parts of file shared"),
N_("not implemented yet(24)")
} ;


// tag_get_realname_from_tagname
//
// expands abbreviated tagname to real name if necessary
//

const gchar *
tag_get_realname_from_tagname (const gchar *tagname)
{
	g_return_val_if_fail (tagname!=NULL, NULL);
	if (strlen(tagname) == 1)
	{
		const guint8 n = (guint8) tagname[0];
		if (n >= (sizeof(TAGNAMES_SHORT_CLEARTEXT)/sizeof(gchar*)))
		{
			return _("not implemented yet");	// some tag value we don't know about
		}
		return TAGNAMES_SHORT_CLEARTEXT[n];
	}
	return tagname;
}

// tag_get_name
//
// returns name of a tag or NULL on error
//
const gchar *
tag_get_name (NGS_tag *tag)
{
	g_return_val_if_fail (tag!=NULL, NULL);
	if (!tag->name) return _("(no name)");
	return tag_get_realname_from_tagname(tag->name);
}


//
//  ---- taglist ----
//
//  a taglist is just a simple GPtrArray* of NGS_tag* records
//

// taglist_write_to_ramfile

void
taglist_write_to_ramfile (GPtrArray *taglist, ramfile *rf)
{
	guint32	i, len;
	g_return_if_fail (rf!=NULL);
	if ((taglist) && ((len=taglist->len)>0))
	{
		ramfile_write32(rf, len); // number of tags
		for (i=0; i<len; i++)
		{
			gpointer p_tag;
			if ((p_tag = g_ptr_array_index(taglist,i)))
			{
				tag_write_to_ramfile ((NGS_tag*)p_tag, rf);
			}
		}
	} else ramfile_write32(rf,0);	// number of tags
}

GPtrArray *
taglist_read_from_ramfile (ramfile *rf)
{
	guint32		no_tags, i;
	GPtrArray	*newtl;
	g_return_val_if_fail (rf!=NULL, NULL);
	no_tags = ramfile_read32(rf);
	newtl = g_ptr_array_new();
	g_return_val_if_fail (newtl!=NULL, NULL);
	for (i=0; i<no_tags; i++)
	{
		gpointer newtag = (gpointer) tag_new_from_ramfile (rf);
		if (!newtag)
		{
			taglist_free(newtl);
			return NULL;
		} else g_ptr_array_add (newtl, newtag);
	}
	return newtl;
}

void
taglist_free (GPtrArray *taglist)
{
	guint	i, len;
	if (!taglist) return;
	len = taglist->len;
	for (i=0; i<len; i++)
	{
		gpointer tag = g_ptr_array_index (taglist,i);
		if (tag) tag_free ((NGS_tag*)tag);
	}
	g_ptr_array_free (taglist, TRUE);
}

/* taglist_dump
 *
 * for debugging: dumps a taglist
 */

void
taglist_dump (GPtrArray *taglist, const gchar *msg)
{
	guint	i, len;
	g_print (N_("dump taglist - %s:\n"), (msg) ? msg : N_(""));
	if ((taglist) && (len=taglist->len)>0)
	{
		for (i=0; i<len; i++)
		{
			gpointer tag = g_ptr_array_index (taglist,i);
			if (tag) tag_dump ((NGS_tag*)tag);
		}
	} else g_print (N_("\t * taglist empty\n"));
}

// taglist_find_tag_from_tagname
//
// find (first) tag in taglist from tagname
// returns tag or NULL if not found

NGS_tag *
taglist_find_tag_from_tagname (GPtrArray *taglist, const gchar *tagname)
{
	guint	i, len;
	g_return_val_if_fail (tagname!=NULL, NULL);
	if (!taglist) return NULL;
	len = taglist->len;
	for (i=0; i<len; i++)
	{
		NGS_tag *tag = (NGS_tag*) g_ptr_array_index (taglist,i);
		if ((tag) && (tag->name))
		{
			if (g_strcasecmp(tag->name, tagname)==0) return tag;
		}
	}
	return NULL;
}


/* tag_string_replace
 *
 * replaces a string tag's string value
 */

void
tag_string_replace(NGS_tag *tag, const gchar *newstring)
{
	g_return_if_fail (tag!=NULL);
	g_return_if_fail (newstring!=NULL);
	g_return_if_fail (tag->type==TAGTYPE_STRING);

	if (tag->data)
		g_free(tag->data);

	tag->data = (gpointer) g_strdup(newstring);

}


/* tag_addedit_string_tag
 *
 * if tag exists:
 *   - mustnotexist == TRUE => does nothing
 *   - mustnotexist == FALSE => change tag to new value
 *
 * if tag does not exist:
 *   - mustexist == TRUE => does nothing
 *   - mustexist == FALSE => adds tag with new value
 *
 */

void
tag_addedit_string_tag (	GPtrArray *taglist,
                        	const gchar *name,
                        	const gchar *newvalue,
                        	gboolean mustexist,
                        	gboolean mustnotexist)
{
	NGS_tag *tag;

	g_return_if_fail (taglist!=NULL);
	g_return_if_fail (name!=NULL);
	g_return_if_fail (newvalue!=NULL);

	tag = taglist_find_tag_from_tagname (taglist, name);

	if (tag)
	{
		if (mustnotexist)
			return;

		if (tag->data)
			g_free(tag->data);

		tag->data = (gpointer) g_strdup(newvalue);
		return;
	}

	if (mustexist)
		return;

	tag = tag_new_string (name, newvalue);
	g_return_if_fail (tag!=NULL);
	g_ptr_array_add(taglist, tag);

	return;
}


/* tag_addedit_integer_tag
 *
 * if tag exists:
 *   - mustnotexist == TRUE => does nothing
 *   - mustnotexist == FALSE => change tag to new value
 *
 * if tag does not exist:
 *   - mustexist == TRUE => does nothing
 *   - mustexist == FALSE => adds tag with new value
 *
 */

void
tag_addedit_integer_tag (	GPtrArray *taglist,
                        	const gchar *name,
                        	guint32 newvalue,
                        	gboolean mustexist,
							gboolean mustnotexist)
{
	NGS_tag *tag;

	g_return_if_fail (taglist!=NULL);
	g_return_if_fail (name!=NULL);

	tag = taglist_find_tag_from_tagname (taglist, name);

	if (tag)
	{
		if (mustnotexist)
			return;

		tag->data = GUINT_TO_POINTER(newvalue);
		return;
	}

	if (mustexist)
		return;

	tag = tag_new_integer (name, newvalue);
	g_return_if_fail (tag!=NULL);
	g_ptr_array_add(taglist, tag);

	return;
}


/* tag_addedit_float_tag
 *
 * if tag exists:
 *   - mustnotexist == TRUE => does nothing
 *   - mustnotexist == FALSE => change tag to new value
 *
 * if tag does not exist:
 *   - mustexist == TRUE => does nothing
 *   - mustexist == FALSE => adds tag with new value
 *
 */

void
tag_addedit_float_tag  (	GPtrArray *taglist,
                        	const gchar *name,
                        	gfloat newvalue,
                        	gboolean mustexist,
                        	gboolean mustnotexist)
{
	NGS_tag *tag;

	g_return_if_fail (taglist!=NULL);
	g_return_if_fail (name!=NULL);

	tag = taglist_find_tag_from_tagname (taglist, name);

	if (tag)
	{
		if (mustnotexist)
			return;

		tag->data = *((gpointer*)&newvalue);
		return;
	}

	if (mustexist)
		return;

	tag = tag_new_float (name, newvalue);
	g_return_if_fail (tag!=NULL);
	g_ptr_array_add(taglist, tag);

	return;
}

