/* pipe.c for ed2k_gui
*
* creates and checks the named pipe opened for receiving ed2k:// links
*
* (c) 2001-2002 Tim-Philipp Muller <t.i.m@orange.net>
*/


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "pipe.h"
#include "http_get.h"
#include "send_command.h"
#include "downloads_page.h"
#include "gui_options.h"
#include "status_page.h"
#include "statusbar.h"
#include "misc.h"
#include "misc_strings.h"

#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>	// XXX - what's this for???
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#define PIPE_BUF_SIZE	256

static gchar		*pipename = NULL;
static GIOChannel	*ioc = NULL;
static guint		 ioc_handler = 0;

static gboolean		pipe_iochannel_callback (GIOChannel *ioc, GIOCondition status, gpointer data);



/* pipe_create
 *
 * create a named pipe to receive ed2k-links etc.
 *
 * returns TRUE if okay, or FALSE if failed
 *
 */

gboolean
pipe_create (void)
{
	gint	pipe_flags;
	gint	npipe;

	if (!pipename)
	{
		pipename = g_strdup(complete_pref_path (N_("pipe"), GUI_CONFIG_USE_INSTANCE));
	}

	g_return_val_if_fail (pipename!=NULL, FALSE);

	/* if pipe exists, delete and create new one to make sure it's a fifo and not a normal file! */
	if (access(pipename, F_OK)==0)
	{
		if (unlink (pipename)!=0)
		{
			gchar	*msg1 = _("GUI: couldn't remove and re-create named pipe to listen for ed2k:// links\n");
			gchar	*msg2 = _("GUI: (possibly owned by other user?) well, it might work anyway...\n");
			status_msg (N_("%s"), msg1);
			status_msg (N_("%s"), msg2);
			statusbar_extra_message (N_(" %s"), msg1);
			return FALSE;
		}
	}

	if (mkfifo(pipename, 0700) < 0)
	{
		status_msg (_("GUI: Could not create named pipe '%s' - %s\n"), pipename, g_strerror(errno));
		return FALSE;
	} /* else status_msg ("GUI: Okay, created named pipe\n"); */

#ifndef FREEBSD
	pipe_flags = O_RDONLY | O_NONBLOCK;
#else
	pipe_flags = O_RDONLY;	/* solves bug #656923 apparently */
#endif

	if ((npipe = open (pipename, pipe_flags)) < 0)
	{
		status_msg (_("GUI: Could not create named pipe '%s' - %s\n"), pipename, g_strerror(errno));
		return FALSE;
	}

	if ((ioc = g_io_channel_unix_new (npipe)))
	{
		static gboolean	 showed_pipefilename = FALSE;
		gchar          	*msg;

		ioc_handler = g_io_add_watch (ioc, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL, pipe_iochannel_callback, NULL);

		if (!showed_pipefilename)
		{
			msg = g_strdup_printf (_("GUI: Listening on named pipe '%s' for ed2k:// links.\n"), pipename);
			g_return_val_if_fail (msg!=NULL, FALSE);
			status_msg (msg);
			g_free(msg);
			showed_pipefilename = TRUE;	/* we don't want this message after every received link */
		}
	} else g_printerr (_("Could not set up GIOChannel on named pipe for ed2k:// links.\n"));
	return (ioc!=NULL);
}


/* pipe_parse_message
 *
 * parses message sent to pipe or message dropped into main window via drag'n'drop
 *
 */

void
pipe_parse_message (gchar *msg)
{
	gchar	*start;

	g_return_if_fail (msg!=NULL);

	/* replace '%xx' unescapes by real characters */
	misc_strings_unescape_url (msg);

	/* Check if it's an ed2k://|command|<advanced command>| link
	 *
	 * XXX - This is 'exploitable' in theory - people could set up
	 *	html-refresh with an ed2k://|command| link - thus the 'q'
	 *	and the 'm # X' commands are not allowed
	 */

	if ((start = strstr (msg, N_("|ipchanged|"))))
	{
		downloads_pause_and_resume_all_active_downloads ();
		status_msg (_("received IP CHANGED indicator\n"));
		return;
	}

	if ((start = strstr (msg, N_("|command|"))))
	{
		gchar *end = strrchr (msg, '|');
		start+=9;
		if (end>start)
		{
			end[0]=0x00;
			while (*start==' ' && start<end) start++;	/* skip initial spaces */
			if (strncmp(start,N_("m "),2)!=0 && start[0]!='q')
			{
				send_command (start);
				status_msg (_("received ed2k:// command link ('%s')\n"), start);
			} else status_msg (_("received ed2k:// command link with not permitted command ('m x y' or 'q')!\n"));
		} else status_msg (_("received invalid ed2k:// command link.\n"));
		return;
	}

	if (extract_ed2k_links (msg) <= 0 )	/* ... and don't clear search results list when extracting the links */
	{
		if (extract_server_links(msg, FALSE) > 0)	/* if not a file link, maybe server links? */
		{
			send_get_serverlist();
			status_msg (_("received ed2k:// server link(s)\n"));
		} else g_printerr (_("received something not an ed2k:// link (via pipe or drag'n'drop)\n"));
	} else status_msg (_("received ed2k:// file link\n"));
}

/* pipe_unlink
 *
 *
 */

void
pipe_unlink (void)
{
	if (ioc)
	{
		g_io_channel_close (ioc);
		g_io_channel_unref (ioc);
	}

	if (ioc_handler>0)
		g_source_remove (ioc_handler);

	ioc = NULL;
	ioc_handler = 0;

	if (pipename)
	{
		unlink (pipename);
//		g_free (pipename);
//		pipename = NULL;
	}
}


/* pipe_iochannel_callback
 *
 *
 */

static gboolean
pipe_iochannel_callback (GIOChannel *ioc, GIOCondition status, gpointer data)
{
	static GIOError	 ret=G_IO_ERROR_NONE;
	static guint	 len=999, size=0;
	static gchar	*buf = NULL;
/*	static gint		 pipe_recreate_count = 0;*/	/* XXX - just to track down bug */

/*	g_print ("%s - condition = %u\n",__FUNCTION__, status); */

	g_return_val_if_fail (ioc!=NULL, FALSE);

	if ( (status&G_IO_IN) == G_IO_IN || (status&G_IO_PRI) == G_IO_PRI )
	{
/*		g_print ("G_IO_IN || G_PRI\n"); */

		while (ret == G_IO_ERROR_NONE && len>0)
		{
			buf = g_renew (gchar, buf, size+PIPE_BUF_SIZE);
			g_return_val_if_fail (buf!=NULL, FALSE);
			ret = g_io_channel_read (ioc, buf+size, PIPE_BUF_SIZE, &len);
			size += len;
/*			g_print ("read from pipe: %u bytes\n", len); */
		}
		// Do not return TRUE here, because status is bit_wise and it could
		//	be G_IO_IN|G_IO_HUP, for instance.
	}

	/* G_IO_OUT should never happen on a read-only pipe */
	g_return_val_if_fail ( (status&G_IO_OUT) == 0, FALSE);


	if (  ((status&G_IO_ERR) == G_IO_ERR || (status&G_IO_HUP) == G_IO_HUP)
		  ||  (((status&G_IO_IN)==G_IO_IN || (status&G_IO_PRI)==G_IO_PRI) && len==0) )
	{
/*		g_print ("G_IO_ERR || G_IO_HUP: (G_IO_ERR=%u) (G_IO_HUP=%u)\n", (status&G_IO_ERR), (status&G_IO_HUP)); */

		pipe_unlink();

/*
#ifdef FREEBSD
		pipe_recreate_count++;
		if (pipe_recreate_count >= 300)
		{
			g_print ("***********************************************\n");
			g_print ("* pipe has been unlinked/re-created more than\n");
			g_print ("* 300 times now. So you've probably\n");
			g_print ("* encountered this bug here:\n");
			g_print ("* http://sourceforge.net/tracker/index.php?func=detail&aid=656923&group_id=61752&atid=498271\n");
			g_print ("* please e-mail the above terminal output to:\n");
			g_print ("* tim@master.vcisp.net\n");
			g_print ("* Thanks!\n");
			g_print ("*\n");
			g_print ("* I will not re-create the pipe now.\n");
			g_print ("***********************************************\n");
			return FALSE;
		}
#endif
*/
		pipe_create();
		if (buf)
		{
/*			g_print ("document size = %u\n\n", size); */
/*			g_print ("%s\n", buf); */

			pipe_parse_message(buf);
			g_free (buf);
			len = 999;
			size = 0;
			ret = G_IO_ERROR_NONE;
			buf = NULL;
		}
		return FALSE;	/* remove event source */
	}

	if ((status&G_IO_NVAL) == G_IO_NVAL)
	{
		g_printerr (_("Named pipe '%s' couldn't be set up (G_IO_NVAL).\n"), pipename);
		pipe_unlink();
		return FALSE;	/* remove event source */
	}

	return TRUE;
}

